/*
Tiedosto: Converge.java
Luokat:   Converge, ConvergeShutdown, PairPanel, UIUpdater, ConvergeAWT

Tekij:   Joni Karppinen
Luonti:   5.12.2002

Muutokset:
  5.12.2002  Joni
              Luokan luonti
  9.12.2002  Olli
              Siirto converge.kernel -pakkauksesta
              converge -pakkaukseen.
              Kyttliittym
  

*/


package converge;
import converge.clientmanager.*;
import converge.attribute.*;
//import converge.service.*;
import converge.kernel.*;
//import java.util.*;
//import org.w3c.dom.*;
import java.awt.*;
import java.awt.event.*;


/**
*
*/
public class Converge{

	
	/**
	*Pohjelmametodi.
  *Mikli parametrin saadaan -gui, kynnistetn vain yllpitjn
  *kyttliittym.
	*Ohjelma voidaan sulkea komennolla ctrl-C, jolloin jrjestelmn
  *alasajo alkaa.
  *
  * @param args komentoriviparametrit
  */
  public static void main(String[] args){    
        ActionLoader.loadClasses();
        AttributeLoader.loadClasses();
        boolean runAll = false;
        UserManager um       = UserManager.getInstance();
        if (args.length == 1 && args[0].equals("-gui"))
          new ConvergeAWT();
        //else {
          runAll = true;
          ViewManager vm       = ViewManager.getInstance();
          ScheduleManager sm   = ScheduleManager.getInstance();
          EventManager em      = EventManager.getInstance();
          ClientManager cm     = ClientManager.getInstance();
        //}
        
        Runtime.getRuntime().addShutdownHook(new ConvergeShutdown(runAll));

	}
  
}

/**
*Tm luokka hoitaa jrjestelmn alasajon.
*
*/
class ConvergeShutdown extends Thread {

  private boolean all = false;
  public ConvergeShutdown(boolean all){
    this.all = all;
  }
  
  public void run() {
    System.out.flush();
    System.err.flush();
    System.out.println("Shutdown...");
    if (all) {
      System.out.println("\tClientManager shutdown");
      ClientManager.getInstance().shutdown();
      System.out.println("\tScheduleManager shutdown");
      ScheduleManager.getInstance().shutdown();
      System.out.println("\tViewManager shutdown");
      ViewManager.getInstance().shutdown();
    }
    System.out.println("\tUserManager shutdown");
    UserManager.getInstance().shutdown();
    System.out.println("\tDatabaseProvider shutdown");
    DatabaseProvider.getInstance().shutdown();
  }

}




/////////////////////////////////////////////////
/// Yllpitjn kyttliittym - versio 0.0.1 /// 
/////////////////////////////////////////////////


/**
*Yhdistetn kaksi paneelia toisiinsa.
*/
class PairPanel extends Panel {

  public PairPanel(Component left, Component right, boolean horiz) {
    if (horiz)
      this.setLayout(new GridLayout(1, 2));
    else
      this.setLayout(new GridLayout(2, 1));
    this.add(left);
    this.add(right);
  }

}


/**
* ConvergeAWT:n pivityksist huolehtiva sie.
*/
class UIUpdater extends Thread {

  private ConvergeAWT awt;
  public UIUpdater(ConvergeAWT awt)  {
    this.awt = awt;
  }

  public void run() {
    UserManager um = UserManager.getInstance();
    try {
    sleep(1000);
    }catch (Exception e) {
      Log.stack(e);
    }
    while (true) {
      try {
      sleep(1000);
      String[] users = um.getUserNames();
      String[] knownUsers = awt.users.getItems();
      java.util.HashSet set = new java.util.HashSet();
      for (int i = 0; i < knownUsers.length; ++i)
        set.add(knownUsers[i]);
      for (int i = 0; i < users.length; ++i)
        if (!set.contains(users[i]))
          awt.users.add(users[i]);
      //awt.users.repaint();
      
      
      String[] groups = um.getGroupNames();
      set = new java.util.HashSet();
      for (int i = 0; i < awt.groups.getItemCount(); ++i)
        set.add(awt.groups.getItem(i));
      for (int i = 0; i < groups.length; ++i)
        if(!set.contains(groups[i])) {
          awt.groups.add(groups[i]);
        }
      //awt.groups.repaint();
      } catch (Exception e) {
        Log.stack(e);
      }
    }
  }
}


/**
*Yllpitjn kyttliittym
*
*/
class ConvergeAWT extends Frame implements WindowListener, ItemListener, ActionListener {
  protected GridLayout gridTop;
  protected Panel userPanel;
  protected Panel groupPanel;
  protected Panel movePanel;
  protected List users;
  protected Choice groups;
  protected List groupUsers;
  
  protected Button deleteUser = new Button("delete user");
  protected Button toGroup = new Button("add to group");
  protected Button addUser = new Button("add user");
  protected TextField userName = new TextField();
  protected TextField userPw = new TextField();
  
  protected Button removeUser = new Button("remove user");
  protected Button deleteGroup = new Button("delete group");
  protected Button addGroup = new Button("add group");
  protected TextField groupName = new TextField();
  protected TextField groupPw = new TextField();
  
  public ConvergeAWT(){
    super("The Converge System");
    this.userPanel = new Panel();
    this.userPanel.setLayout(new BorderLayout());
    this.groupPanel = new Panel();
    this.groupPanel.setLayout(new BorderLayout());
    this.setLayout(new BorderLayout());
    this.add(userPanel);
    this.add(groupPanel);
    this.groups = new Choice();
    this.groups.add("");
    this.groups.addItemListener(this);
    this.groupUsers = new List();
    this.users = new List();
    this.add(new PairPanel( new Label("Users:"),
                  new PairPanel(new Label("Groups:"), groups, true), true), 
                                                        BorderLayout.NORTH);
    Panel leftbottom = new Panel();
    leftbottom.setLayout(new GridLayout(5, 1));
    Panel rightbottom = new Panel();
    rightbottom.setLayout(new GridLayout(5, 1));
    deleteUser.addActionListener(this);
    leftbottom.add(deleteUser);
    toGroup.addActionListener(this);
    leftbottom.add(toGroup);
    leftbottom.add(new PairPanel(new Label("username"), userName, true));
    userPw.setEchoChar('*');
    leftbottom.add(new PairPanel(new Label("password"), userPw, true));
    addUser.addActionListener(this);
    leftbottom.add(addUser);
    
    removeUser.addActionListener(this);
    rightbottom.add(removeUser);
    deleteGroup.addActionListener(this);
    rightbottom.add(deleteGroup);
    rightbottom.add(new PairPanel(new Label("groupname"), groupName, true));
    groupPw.setEchoChar('*');
    rightbottom.add(new PairPanel(new Label("password "), groupPw, true));
    addGroup.addActionListener(this);
    rightbottom.add(addGroup);
    this.add(new PairPanel(this.users,this.groupUsers, true), 
                                                        BorderLayout.CENTER);
    this.add(new PairPanel(leftbottom, rightbottom, true), 
                                                        BorderLayout.SOUTH);
    
    this.addWindowListener(this); 
    this.pack();
    this.setVisible(true);
    new UIUpdater(this).start();
  }
  
  
  public void itemStateChanged(ItemEvent e) {
    this.groupUsers.removeAll();

    String name = this.groups.getSelectedItem();
    if (name == null)
      return;
    Group group = UserManager.getInstance().getGroup(name);
    if (group == null)
      return;
    String[] userNames = group.getUserNames().getUserNames();
    for (int i = 0; i < userNames.length; ++i)
      this.groupUsers.add(userNames[i]);
  }
  
  public void actionPerformed(java.awt.event.ActionEvent e) {
    if (e.getSource() == this.addUser && this.userName.getText().length() > 0 &&
                                           this.userPw.getText().length() > 0) {
      User user = UserManager.getInstance().newUser(this.userName.getText(), 
                                                    this.userPw.getText());
      this.userName.setText("");
      this.userPw.setText("");
    }
    else if (e.getSource() == this.addGroup && 
                                    this.groupName.getText().length() > 0 &&
                                    this.groupPw.getText().length() > 0) {
      Group group = UserManager.getInstance().newGroup(this.groupName.getText(), 
                                                        this.groupPw.getText());
      this.groupName.setText("");
      this.groupPw.setText("");
      this.groups.select("");
    } else if (e.getSource() == this.toGroup && 
                                this.groups.getSelectedItem() != null &&
                                this.groups.getSelectedItem().length() > 0 &&
                                this.users.getSelectedItem() != null &&
                                this.users.getSelectedItem().length() > 0) {
        //System.out.println("HERE: [" + this.groups.getSelectedItem() + "]");
        Group group = UserManager.getInstance().getGroup(
                                              this.groups.getSelectedItem());
        if (group == null)
          return;
        //System.out.println("Group found " + group.getName());
        this.groupUsers.removeAll();
        group.writeLock(this);
        group.getUserNames().addUser(this.users.getSelectedItem());
        group.releaseLock(this);
        String[] userNames = group.getUserNames().getUserNames();
        for (int i = 0; i < userNames.length; ++i)
          this.groupUsers.add(userNames[i]);
    }
    else if (e.getSource() == this.removeUser && 
                                this.groups.getSelectedItem() != null &&
                                this.groups.getSelectedItem().length() > 0 &&
                                this.groupUsers.getSelectedItem() != null &&
                                this.groupUsers.getSelectedItem().length() > 0) {
        //System.out.println("HERE: [" + this.groups.getSelectedItem() + "]");
        Group group = UserManager.getInstance().getGroup(
                                                this.groups.getSelectedItem());
        if (group == null)
          return;
        //System.out.println("Group found " + group.getName());
        group.writeLock(this);
        group.getUserNames().removeUser(this.groupUsers.getSelectedItem());
        group.releaseLock(this);
        this.groupUsers.removeAll();
        String[] userNames = group.getUserNames().getUserNames();
        for (int i = 0; i < userNames.length; ++i)
          this.groupUsers.add(userNames[i]);
    }
    else if (e.getSource() == this.deleteUser && 
           this.users.getSelectedItem() != null &&
           this.users.getSelectedItem().length() > 0) {
      UserManager.getInstance().deleteUser(this.users.getSelectedItem());
      this.users.remove(this.users.getSelectedItem());
      this.groupUsers.removeAll();
      this.groups.select(this.groups.getSelectedItem());
    }
    else if (e.getSource() == this.deleteGroup && 
           this.groups.getSelectedItem() != null &&
           this.groups.getSelectedItem().length() > 0) {
      UserManager.getInstance().deleteGroup(this.groups.getSelectedItem());
      this.groups.remove(this.groups.getSelectedItem());
      this.groups.select("");
    }
    
  
  } 
  public void windowClosed(WindowEvent e) {
    System.exit(0);
  } 
  public void windowDeactivated(WindowEvent e) {} 
  public void windowActivated(WindowEvent e) {} 
  public void windowDeiconified(WindowEvent e) {} 
  public void windowIconified(WindowEvent e) {} 
  public void windowOpened(WindowEvent e) {} 
  public void windowClosing(WindowEvent e){
    System.exit(0);
  }
}
