/*

Tiedosto   : AttributeLoader.java
Luokat     : AttributeLoader

         
Tekij     : Olli Pettay
Luonti pvm : 19.11.2002


Muutokset :

19.11.2002 Olli Pettay
           Luotu
4.12.2002  Mikko Hiipakka
           Listty UserProperty aksessori

7.12.2002 Olli Pettay
          getKnown* -metodit

*/


package converge.attribute;

import converge.kernel.Log;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;


/**
 * Palvelu luokka, jonka avulla voidaan pyyt attribuutti-ilmentymi 
 * UserPropertyInterface:lle, ContextModelAttributeInterface:lle tai MessageAttributeInterface:lle.
 * Uutta ilmentym pyydetn attribuutin loogisella nimell. Nm nimet voidaan muutosten jlkeen 
 * pivitt luokkaan loadClasses-metodilla 
*/ 
public class AttributeLoader{
  
  private static String packageName         = "converge.attribute.attributeImplementations.";
  private static final String attributeDir  = "converge.attribute.dir";
  private static String defaultAttributeDir = "converge"
                                             + File.separator
                                             + "attribute"
                                             + File.separator
                                             + "attributeImplementations"
                                             + File.separator;
                                             
  private static HashMap messageAttrs      = new HashMap();   //MessageAttributes
  private static HashMap contextModelAttrs = new HashMap();   //ContextModelAttributes
  private static HashMap userProperties    = new HashMap();   //UserProperties
  


  /**
   * Palveluna luodaan UserPropertyInterfacen ilmentym, joka
   * tarjoaa toteutuksen "name"-nimiselle attribuutille
   * @param name Pyydetyn attribuuttin looginen nimi
   * @return Palauttaa uuden UserPropertyInterface ilmentymn, ja
   * jos luonti eponnistuu palauttaa null
  */ 
  public static UserPropertyInterface newUserPropertyInstance(String name) {
    Class aClass = (Class)userProperties.get(name);
    UserPropertyInterface toReturn = null;
    if (aClass != null) {
      try{
        toReturn = (UserPropertyInterface)aClass.newInstance();
      }catch( Exception ex ){
        Log.stack( ex );
      }
    }
    return toReturn;
  }


  
  
  /**
   * Palveluna luodaan ContextModelAttributeInterface ilmentym, joka
   * tarjoaa toteutuksen "name"-nimiselle attribuutille
   * @param name Pyydetyn attribuuttin looginen nimi
   * @return Palauttaa uuden ContextModelAttributeInterface ilmentymn, ja
   * jos luonti eponnistuu palauttaa null
  */ 
  public static ContextModelAttributeInterface newContextModelAttributeInstance(String name) {
    Class aClass = (Class)contextModelAttrs.get(name);
    ContextModelAttributeInterface toReturn = null;
    if (aClass != null) {
      try{
        toReturn = (ContextModelAttributeInterface)aClass.newInstance();
      }catch( Exception ex ){
        Log.stack( ex );
      }
    }
    return toReturn;
  }


  /**
   * Palveluna luodaan MessageAttributeInterface ilmentym, joka
   * tarjoaa toteutuksen "name"-nimiselle attribuutille
   * @param name Pyydetyn attribuuttin looginen nimi
   * @return Palauttaa uuden MessageAttributeInterface ilmentymn, ja
   * jos luonti eponnistuu palauttaa null
  */   
  public static MessageAttributeInterface newMessageAttributeInstance(String name) {
    Class aClass = (Class)messageAttrs.get(name);
    MessageAttributeInterface toReturn = null;
    if (aClass != null) {
      try{
        toReturn = (MessageAttributeInterface)aClass.newInstance();
      }catch( Exception ex ){
        Log.stack( ex );
      }
    }
    return toReturn;
  }
  
  


  /**
   * Ladataan Attribuutti toteutusluokat, ja tarkastetaan niiden tarjoamat 
   * toteutukset MessageAttributeInterface:lle,ContextModelAttributeInterface:lle 
   * ja UserPropertyInterface:lle
  */     
  public static synchronized void loadClasses() {
    //Ymprist muutujanarvo tai default arvo
    String pathToAttributes = System.getProperty(attributeDir, defaultAttributeDir);
    File dir = new File(pathToAttributes);
    
    if (!dir.isDirectory()) {
     Log.err.println("AttributeLoader-loadClasses - Ei ole hakemisto : " + dir.getAbsolutePath() );
      return; 
    }
    messageAttrs = new HashMap();
    contextModelAttrs = new HashMap();
    userProperties = new HashMap();

    File[] classes = dir.listFiles(new FilenameFilter(){
              public boolean accept(File dir, String name) {
                if (name.endsWith(".class")) 
                  return true;
                return false;
              }
            }); 
    for (int i = 0; i < classes.length; ++i) {
      String name = classes[i].getName();
      String classname = name.substring(0,name.indexOf(".class"));  
      try{
        Class aClass = Class.forName( packageName + classname);
        //DEBUG
        Log.output("AttributeLoader").println("\nATTRIBUTELOADER-loadClasses : Ladataan luokka : " + classname );
        //DEBUG
        setAttributes( aClass );
      }catch( ClassNotFoundException ex ){
        Log.stack( ex );
      }

    }
  }
  
  
  private static void setAttributes( Class aClass ) {
    Object o = null;
    try{
      o = aClass.newInstance();
    }catch( Exception ex ){
      Log.stack( ex );
      return; 
    }
    if ( o instanceof AttributeInterface )
      setAttributeInterface( (AttributeInterface)o, aClass );
    else if( o instanceof UserPropertyInterface )
      setUserProperty( (UserPropertyInterface)o, aClass );
  }
  
  
  
   private static void setAttributeInterface( AttributeInterface attr, Class aClass ){
    String[] supported = attr.supportedAttributes();
    if ( supported == null )
      return;
    for(int i = 0; i < supported.length; ++i) {
      if (attr instanceof MessageAttributeInterface) {
        if ( !messageAttrs.containsKey(supported[i]) )
            messageAttrs.put(supported[i], aClass);
        else{
          Class old = (Class)messageAttrs.get(supported[i]);
          Log.output("AttributeLoader").println("REDEFINING MessageAttributeInterface:" 
                                             + supported[i]
                                             + ".  " + old.getName() 
                                             + " : " + aClass.getName());
        }
      }
      if ( attr instanceof ContextModelAttributeInterface) {
        if ( !contextModelAttrs.containsKey(supported[i]) ) 
          contextModelAttrs.put(supported[i], aClass);
        else{
          Class old = (Class)messageAttrs.get(supported[i]);
          Log.output("AttributeLoader").println("REDEFINING ContextModelAttributeInterface:" 
                                             + supported[i]
                                             + ".  " + old.getName() 
                                             + " : " + aClass.getName());
        }
      }
    }
    //DEBUG********************************************************************
    Log.output("AttributeLoader").println("Supported");
    for(int i = 0; i < supported.length; ++i) 
      Log.output("AttributeLoader").println("  -  " +supported[i]);
    //DEBUG********************************************************************
  }
  
  
  
  
  private static void setUserProperty( UserPropertyInterface attr, Class aClass ){
    //DEBUG********************************************************************
    Log.output("AttributeLoader").print("UserPropertyInterface");
    //DEBUG********************************************************************
    String[] supported = attr.supportedAttributes();
    if ( supported == null )
      return;
    for(int i = 0; i < supported.length; ++i) {
      if ( !userProperties.containsKey(supported[i]) ) 
        userProperties.put(supported[i], aClass);
      else{
          Class old = (Class)messageAttrs.get(supported[i]);
          Log.output("AttributeLoader").println("REDEFINING UserPropertyInterface:" 
                                             + supported[i]
                                             + ".  " + old.getName() 
                                             + " : " + aClass.getName());
      }
    }
    //DEBUG***********************************************************************
    Log.output("AttributeLoader").println("Supported");
    for(int i = 0; i < supported.length; ++i) 
      Log.output("AttributeLoader").println("  -  " + supported[i]);
    //DEBUG**********************************************************************
  } 
  
  // Subject => AttributeString; Date => AttributeDate
  public static String[] getKnownMessageAttributes() {
    return (String[]) messageAttrs.keySet().toArray(new String[0]);
  }
  
  public static String[] getKnownContextModelAttributes() {
    return (String[]) contextModelAttrs.keySet().toArray(new String[0]);
  }
  
  public static String[] getKnownUserProperties() {
    return (String[]) userProperties.keySet().toArray(new String[0]);
  }
  
//DEBUG************************************************************************
  public static void main(String[] args){
    AttributeLoader.loadClasses();
    MessageAttributeInterface mai = newMessageAttributeInstance( "sender" );
    String[] supported = mai.supportedAttributes();
    System.out.println("");
    System.out.println("");
    System.out.println("Main attribute supports");
    for(int i = 0; i < supported.length; ++i) 
      System.out.println("  -  " +supported[i]);
      
    String[] all = getKnownMessageAttributes();
    System.out.println("Known MessageAttributes:");
    for (int i = 0; i < all.length; ++i)
      System.out.println("Attribute:" + all[i]);
      
    all = getKnownContextModelAttributes();
    System.out.println("Known ContextModelAttributes:");
    for (int i = 0; i < all.length; ++i)
      System.out.println("Attribute:" + all[i]);
      
    all = getKnownUserProperties();
    System.out.println("Known UserProperties:");
    for (int i = 0; i < all.length; ++i)
      System.out.println("Property:" + all[i]);
  }
//DEBUG************************************************************************
}
