/*
Tiedosto: AttributeDate.java
Luokat:   AttributeDate

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti
  19.12.2002  Testausta ja siistimist
*/

package converge.attribute.attributeImplementations;

import converge.service.Message;
import converge.attribute.Attribute;
import java.util.Date;
import org.w3c.dom.*;
import converge.kernel.Log;

/**
* Attribute-luokan aliluokka, joka osaa ksitell pivmri (esim. viestin lhetysaika)
*/
public class AttributeDate extends Attribute{

    private Message myMsg;
    private Date myValue = new Date();
    private String myValueString = "";
    private static String[] supported = { "Date" };
    private static String description = "Pivmrtietojen ksittely";

    /**
    * Vertaa olion arvoa annetulla tavalla annettuun objektiin.
    *
    * @param expType vertailutyyppi
    * @param compareValue vertailtava Date-olio
    * @return vertailun tulos
    */
    public boolean compare(String expType, Object compareValue) {
        parseMyValue();
        if(compareValue == null || myValueString == null)
            return false;

        if(compareValue instanceof String) {
            try {
                AttributeString as = new AttributeString();
                as.setValue(myValueString);
                return as.compare(expType,compareValue);
            } catch(Exception e) {
                return false;
            }
        }

        if(myValue == null)
            return false;

        Date givenValue = null;
        try {
            givenValue = (Date)compareValue;
        } catch(Exception e) {
            return false;
        }

        try {
            if(expType.equals("="))
                return this.myValue.compareTo(givenValue) == 0;

            if(expType.equals("!="))
                return this.myValue.compareTo(givenValue) != 0;

            if(expType.equals("<"))
                return this.myValue.compareTo(givenValue) < 0;

            if(expType.equals(">"))
                return this.myValue.compareTo(givenValue) > 0;
        } catch(Exception e) {
            return false;
        }

        return false;
    }


    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public boolean compare(converge.service.Message msg) {
        if(this.value == null)
            return false;
        parseMyValue();
        if(myValue == null && myValueString == null)
            return false;

        if(this.name == null)
            this.name=supported[0];

        if(myValue == null) { //vertaillaan merkkijonona
            try {
                AttributeString as = new AttributeString();
                as.setValue(msg.getMessageAttribute(this.name).getValue().toString());
                return as.compare(this.expressionType,myValueString);
            } catch(Exception e) {
                return false;
            }
        }

        try { // Date-olioiden vertailu
            Attribute a = (Attribute)msg.getMessageAttribute(this.name);
            if(a == null)
                return false;
            return a.compare(this.expressionType,myValue);
        } catch(Exception e) {
            return false;
        }
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public String[] supportedAttributes() {
        return supported;
    }

    /**
    * Muodostaa AttributeDate-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        if(parent==null)
            return;
        parseMyValue();

        Document doc;
        if(parent.getOwnerDocument() == null)
            doc = (Document)parent;
        else
            doc = parent.getOwnerDocument();
        Element ad = doc.createElement("AttributeDate");
        parent.appendChild(ad);

        Element name = doc.createElement("name");
        if(this.name == null)
            this.name=supported[0];

        name.appendChild(doc.createTextNode(this.name));
        ad.appendChild(name);

        Element value = doc.createElement("value");
        if(this.value == null) {
            System.out.println("AttributeDate: date is null");
            value.appendChild(doc.createTextNode("0"));
        } else if(this.value instanceof Date)
            value.appendChild(doc.createTextNode(new Long(((Date)this.value).getTime()).toString()));
        else if(this.value instanceof String)
            value.appendChild(doc.createTextNode((String)this.value));
        else
            value.appendChild(doc.createTextNode("0"));

        ad.appendChild(value);
    }

    /**
    * Muodostaa annetusta XML-elementist AttributeDate-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        if ( myContent == null || !myContent.getNodeName().equals("AttributeDate") )
            return false;
        try {
          Node curr = myContent.getFirstChild();
          while(curr != null) {
              if(curr.getNodeName().equals("name"))
                  this.name = curr.getFirstChild().getNodeValue();
              else if(curr.getNodeName().equals("value")) {
                  if(curr.getFirstChild().getNodeValue().equals("0"))
                      this.value=null;
                  else
                      try {
                          this.value=new Date(new Long(curr.getFirstChild().getNodeValue()).longValue());
                      } catch(Exception datee) {
                          this.value=curr.getFirstChild().getNodeValue();
                      }
              }
              curr=curr.getNextSibling();
          }
        } catch(Exception ex) {
          Log.output("AttributeDate").println("XML lataus ei onnistunut");
          Log.output("AttributeDate").println(ex.getMessage());
          Log.stack(ex);
          return false;
        }
        return this.name != null && this.value != null;
    }

    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public String getDescription(){
        return description;
    }

    /**
    * Tarkistaa olion sisltmn tietoarvon ja muuttaa sen normaalimuotoon
    */
    private void parseMyValue(){
        try {
            myValue = (Date)this.value;
            myValueString = myValue.toString();
        } catch(Exception ex){
            try {
                myValue = null;
                myValueString = (String)this.value;
            } catch(Exception ex2) {
                myValue=null;
                myValueString=null;
                Log.stack(ex2,"AttributeDate");
            }
        }
    }

    /**
    * Asettaa attribuutin sisltmn viestiolion viitteen
    *
    * @param msg Jrjestelmn sisinen viesti
    */
    public void setMessage(Message msg){
        this.myMsg = msg;
    }

    /* testimetodi */
    public static void main(String[] args) {
        System.out.println("AttributeDate: alkuperinen a1");
        org.w3c.dom.Document doc = converge.kernel.XMLUtil.newXMLDocument();

        AttributeDate a1 = new AttributeDate();
        a1.setValue(new Date());
        a1.createXML(doc);
        converge.kernel.XMLUtil.outputPrinting(doc,System.out);

        System.out.println("AttributeDate: kopio a2");
        AttributeDate a2 = new AttributeDate();
        a2.loadXML(doc.getFirstChild());
        org.w3c.dom.Document doc2 = converge.kernel.XMLUtil.newXMLDocument();
        a2.createXML(doc2);
        converge.kernel.XMLUtil.outputPrinting(doc2,System.out);

        System.out.println("AttributeDate: alkuperinen b1");
        org.w3c.dom.Document docb1 = converge.kernel.XMLUtil.newXMLDocument();

        AttributeDate b1 = new AttributeDate();
        b1.setValue("Sat");
        b1.createXML(docb1);
        converge.kernel.XMLUtil.outputPrinting(docb1,System.out);

        System.out.println("AttributeDate: kopio b2");
        AttributeDate b2 = new AttributeDate();
        b2.loadXML(doc.getFirstChild());
        org.w3c.dom.Document docb2 = converge.kernel.XMLUtil.newXMLDocument();
        b2.createXML(docb2);
        converge.kernel.XMLUtil.outputPrinting(docb2,System.out);

        System.out.println("AttributeDate: vanhempi pvm a3");
        AttributeDate a3 = new AttributeDate();
        org.w3c.dom.Document doc3 = converge.kernel.XMLUtil.newXMLDocument();
        a3.setValue(new Date((new Date()).getTime()-10000));
        a3.createXML(doc3);
        converge.kernel.XMLUtil.outputPrinting(doc3,System.out);

        System.out.println("a1 = a2 : " + a1.compare("=",a2.getValue()));
        System.out.println("a1 = a3 : " + a1.compare("=",a3.getValue()));
        System.out.println("a2 = a3 : " + a2.compare("=",a3.getValue()));
        System.out.println("a1 < a2 : " + a1.compare("<",a2.getValue()));
        System.out.println("a1 < a3 : " + a1.compare("<",a3.getValue()));
        System.out.println("a2 < a3 : " + a2.compare("<",a3.getValue()));
        System.out.println("a1 > a2 : " + a1.compare(">",a2.getValue()));
        System.out.println("a1 > a3 : " + a1.compare(">",a3.getValue()));
        System.out.println("a2 > a3 : " + a2.compare(">",a3.getValue()));
        System.out.println("a1 = Sat : " + a1.compare("=","Sat"));
        System.out.println("a1 != Sat : " + a1.compare("!=","Sat"));
        System.out.println("a1 in Sat : " + a1.compare("in","Sat"));

        /* tulosti:
        a1 == a2 : true
        a1 == a3 : false
        a2 == a3 : false
        a1 < a2 : false
        a1 < a3 : false
        a2 < a3 : false
        a1 > a2 : false
        a1 > a3 : true
        a2 > a3 : true
        a1 = Sat : false
        a1 != Sat : true
        a1 in Sat : true    riippuu tietenkin pivst
        */
    }
}
