/*
Tiedosto: AttributePrecedence.java
Luokat:   AttributePrecedence

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti
  18.12.2002  Toiminnallisuutta
*/

package converge.attribute.attributeImplementations;

import converge.kernel.Log;
import converge.service.Message;
import converge.attribute.Attribute;
import org.w3c.dom.*;

/**
* Attribute-luokan aliluokka, joka osaa ksitell viestin mukana tullutta
* Precedence-arvoa (junk, bulk, normal, first-class)
*/
public class AttributePrecedence extends Attribute {

    private Message myMsg;
    private Integer myValue = new Integer(2); // vastaa arvoa "normal"
    private static String[] supported = { "Precedence" };
    private static String description = "Viestin Precedence-kentn ksittely (trkeys)";

    /**
    * Vertaa olion arvoa annetulla tavalla annettuun objektiin.
    *
    * @param expType vertailutyyppi
    * @param compareValue vertailtava olio
    * @return vertailun tulos
    */
    public boolean compare(String expType, Object compareValue) {
        parseMyValue();
        if(compareValue == null || myValue == null)
            return false;
        Integer givenValue = new Integer(2);
        try {
            compareValue = (String)compareValue;
            if(compareValue.equals("junk") || compareValue.equals("0"))
                compareValue="0";
            else if(compareValue.equals("bulk") || compareValue.equals("1"))
                compareValue="1";
            else if(compareValue.equals("first-class") || compareValue.equals("3"))
                compareValue="3";
            else
                compareValue="2";
            givenValue = new Integer((String)compareValue);
        } catch(Exception ex) {
            Log.stack(ex,"AttributePrecedence");
            return false;
        }
        try {
            if (expType.equals("="))
                return this.myValue.compareTo(givenValue) == 0;
            if (expType.equals("!="))
                return this.myValue.compareTo(givenValue) != 0;
            if (expType.equals("<"))
                return this.myValue.compareTo(givenValue) < 0;
            if (expType.equals(">"))
                return this.myValue.compareTo(givenValue) > 0;
        } catch(Exception e) {
            return false;
        }
        return false;
    }

    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public boolean compare(converge.service.Message msg) {
        parseMyValue();
        if(myValue == null)
            return false;
        if(this.name == null)
            this.name=supported[0];
        try {
            Attribute a = (Attribute)msg.getMessageAttribute(this.name);
            if(a == null)
                return false;
            return a.compare(this.expressionType,myValue.toString());
        } catch(Exception e) {
            return false;
        }
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public String[] supportedAttributes() {
        return supported;
    }

    /**
    * Muodostaa AttributePrecedence-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        if(parent==null)
            return;
        parseMyValue();

        Document doc;
        if(parent.getOwnerDocument() == null)
            doc = (Document)parent;
        else
            doc = parent.getOwnerDocument();

        Element apre = doc.createElement("AttributePrecedence");
        parent.appendChild(apre);

        Element name = doc.createElement("name");
        name.appendChild(doc.createTextNode(this.name));
        apre.appendChild(name);

        Element ex = doc.createElement("expression");
        ex.appendChild(doc.createTextNode(this.expressionType));
        apre.appendChild(ex);

        Element value = doc.createElement("value");
        value.appendChild(doc.createTextNode(this.value.toString()));
        apre.appendChild(value);
    }

    /**
    * Muodostaa annetusta XML-elementist AttributePrecedence-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        if ( myContent == null || !myContent.getNodeName().equals("AttributePrecedence") ) {
            return false;
        }
        try {
            Node curr = myContent.getFirstChild();
            while(curr != null) {
                if(curr.getNodeName().equals("name")) {
                    this.name = curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("value")) {
                    this.value=curr.getFirstChild().getNodeValue();
                }
                else if(curr.getNodeName().equals("expression")) {
                    this.expressionType=curr.getFirstChild().getNodeValue();
                }

                curr=curr.getNextSibling();
            }
        } catch(Exception ex) {
            Log.output("AttributePrecedence").println("XML lataus ei onnistunut");
            Log.output("AttributePrecedence").println(ex.getMessage());
            return false;
        }
        return this.name != null && this.value != null;
    }

    /**
    * Tarkistaa olion sisltmn tietoarvon ja muuttaa sen normaalimuotoon
    */
    private void parseMyValue(){
        if(this.value == null) {
            this.value="2"; // oletusarvo, "normaali"
            myValue=new Integer("2");
        }
        try{
            String compareValue = (String)this.value;
            if(compareValue.equals("junk") || compareValue.equals("0"))
                compareValue="0";
            else if(compareValue.equals("bulk") || compareValue.equals("1"))
                compareValue="1";
            else if(compareValue.equals("first-class") || compareValue.equals("3"))
                compareValue="3";
            else
                compareValue="2";
            myValue = new Integer((String)compareValue);
        } catch(Exception ex) {
            Log.stack(ex,"AttributePrecedence");
        }
    }

    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public String getDescription() {
        return description;
    }

    /**
    * Asettaa attribuutin sisltmn viestiolion viitteen
    *
    * @param msg Jrjestelmn sisinen viesti
    */
    public void setMessage( Message msg) {
        this.myMsg = msg;
    }

    /* testimetodi */
    public static void main(String[] args) {
        AttributePrecedence a = new AttributePrecedence();
        a.setValue("3"); //first-class
        System.out.println(" 3 = 3 => " + a.compare("=","3"));
        System.out.println(" 3 = 2 => " + a.compare("=","2"));
        System.out.println(" 3 > 4 => " + a.compare(">","4"));
        System.out.println(" 3 < 4 => " + a.compare("<","4"));
        a.setValue("xxx"); //virhe, vastaa kakkosta
        System.out.println(" xxx = 3 => " + a.compare("=","3"));
        System.out.println(" xxx = 2 => " + a.compare("=","2"));
        System.out.println(" xxx > 1 => " + a.compare(">","1"));
        System.out.println(" xxx > 4 => " + a.compare(">","4"));
        System.out.println(" xxx < 4 => " + a.compare("<","4"));
        System.out.println(" xxx = 4 => " + a.compare("=","4"));
        /* tulostaa:
        3 = 3 => true       ok
        3 = 2 => false      ok
        3 > 4 => true       ok, 4 on laiton arvo, muutetaan 2:ksi => 3>2 = true
        3 < 4 => false      ok, 4 on laiton arvo, muutetaan 2:ksi => 3<2 = false

                            xxx on laiton arvo, muutetaan 2:ksi
        xxx = 3 => false    ok, 2=3 = false
        xxx = 2 => true     ok, 2=2 = true
        xxx > 1 => true     ok, 2>1 = true
        xxx > 4 => false    ok, 4 on laiton arvo, muutetaan 2:ksi => 2>2 = false
        xxx < 4 => false    ok, 4 on laiton arvo, muutetaan 2:ksi => 2<2 = false
        xxx = 4 => true     ok, molemmat laittomia => 2=2 = true
        */
    }
}
