/*

Tiedosto   : StringArrayImplementation.java
Luokat     : StringArrayImplementation

Tekij     : Mikko Hiipakka
Luonti pvm : 23.11.2002


Muutokset :

23.11.2002 Mikko Hiipakka
           Luotu
19.12.2002 Anssi Johansson
           Kommentointi
*/


package converge.attribute.attributeImplementations;

import converge.kernel.Log;
import converge.service.Message;
import converge.attribute.Attribute;
import org.w3c.dom.*;

/**
 * Attribuuttiluokka, jonka avulla voidaan ksitell taulukkotyyppisi muuttujia.
 * Tll hetkell kyttmtn luokka, voitaisiin kytt jatkossa esim. viestin
 * vastaanottajien selvittmisess. Voi vaatia viel tyt ennen kyttnottoa..
*/
public class AttributeStringArray extends Attribute {

    private Message myMsg;
    private static String description = "Attribuutti voi sislt useamman tekstityyppisi arvon";
    private static String[] supported = {"sender"};//null;
    private String[] myValue;

    /**
    * Vertaa olion arvoa annettuun jrjestelmn sisisen viestin vastaavaan arvoon.
    *
    * @param msg vertailtava viesti
    * @return vertailun tulos
    */
    public boolean compare ( converge.service.Message msg ) {
        return false;
    }

    /**
    * Vertaa olion arvoa annetulla tavalla annettuun objektiin.
    *
    * @param expType vertailutyyppi
    * @param compareValue vertailtava olio
    * @return vertailun tulos
    */
    public boolean compare ( String expressionType, Object givenValue ) {
        parseMyValue();
        String compareValue = null;
        try {
            compareValue = (String)givenValue;
        } catch(Exception ex){
            Log.stack( ex );
            return false;
        }

        if ( expressionType.equals( "in" ) ) {
            for( int j=0; j<this.myValue.length; j++ ) {
                if( myValue[j].equals( compareValue ) )
                    return true;
            }
            return false;
        }
        if ( expressionType.equals( "!in" ) ) {
            for( int j=0; j<this.myValue.length; j++ ) {
                if( myValue[j].equals( compareValue ) )
                    return false;
            }
            return true;
        }

        if ( expressionType.equals( "=" ) ) {
            int count = 0;
                for( int j=0; j<this.myValue.length; j++ ) {
                    if( myValue[j].equals( compareValue ) )
                        count++;
                }
            return ( count == myValue.length );
        }
        return false;
    }

    /**
    * Palauttaa taulukon tmn luokan tukemista attribuuttien nimist.
    *
    * @return attribuuttien nimien taulukko
    */
    public String[] supportedAttributes() {
        //return supported;
        return new String[0];
    }

    /**
    * Muodostaa AttributeStringArray-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        if(parent==null)
            return;
        parseMyValue();
        Document dom = parent.getOwnerDocument();
        Element root = dom.createElement("AttributeStringArray");
        parent.appendChild( root );
        Element myContent = dom.createElement(this.name);
        String a = "" + this.myValue.length;
        myContent.setAttribute("length", a );
        myContent.setAttribute("expressionType", this.expressionType );
        root.appendChild( myContent );
        for( int i=0; i<this.myValue.length; i++) {
            Element item = dom.createElement("data");
            item.setAttribute("value", this.myValue[i] );
            myContent.appendChild(item);
        }
    }

    /**
    * Muodostaa annetusta XML-elementist AttributeStringArray-olion
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        Element root =  (Element)myContent;
        if ( !(root.getTagName()).equals("AttributeStringArray") ){
            return false;
        }
        Element me = (Element)root.getFirstChild();
        this.name= me.getTagName();
        String[] v = new String[ Integer.parseInt(me.getAttribute("length")) ];
        this.expressionType = me.getAttribute("expressionType");
        Element data = (Element)me.getFirstChild();
        int count = 0;
        while ( data != null ){
            v[count] = data.getAttribute("value");
            data = (Element)data.getNextSibling();
            count++;
        }
        this.value = v;
        return true;
    }

    /**
    * Palauttaa tmn attribuuttityyppiluokan kuvauksen.
    *
    * @return luokan kuvaus
    */
    public String getDescription(){
        return description;
    }

    /**
    * Asettaa attribuutin sisltmn viestiolion viitteen
    *
    * @param msg Jrjestelmn sisinen viesti
    */
    public void setMessage( Message msg){
        this.myMsg = msg;
    }

    /**
    * Tarkistaa olion sisltmn tietoarvon ja muuttaa sen normaalimuotoon
    */
    private void parseMyValue(){
        try {
            this.myValue = (String[])value;
        } catch(Exception ex){
            Log.stack(ex);
            this.myValue = new String[0];
        }
    }

    //DEBUG
    public static void main(String[] args){

    }
    //DEBUG
}

