/*
Tiedosto: ContactList.java
Luokat:   ContactList

Tekij:   Olli Pettay
Luonti:   24.11.2002

Muutokset:
  24.11.2002  Luokan luonti

*/


package converge.kernel;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xindice.xml.TextWriter;

/**
*Kontaktilistat voivat sislt toisia kontaktilistoja ja varsinaisia 
*kontaktitietoja.
*/
public class ContactList implements XMLContentInterface{
  private String name = null;
  private HashMap contacts;
  private HashMap contactlists;
  public ContactList(String name){
    this.name = name;
    this.contacts = new HashMap();
    this.contactlists = new HashMap();
  
  }
  
  public ContactList() {
    this(null);
  }
  
  public String getName() {
    return this.name;
  }
  
  /**
  *Kontaktilistan kontaktietojen nimet.
  */
  public Set getContactNames(){
    return this.contacts.keySet();
  }
  
  /**
  *@param name nimi
  *@return palautetaan kontaktitieto nimen perusteella 
  */
  public Contact getContact(String name){
    return (Contact) this.contacts.get(name);
  }
  
  /**
  *Kontaktilistan alikontaktilistojen nimet.
  */
  public Set getContactListNames(){
    return this.contactlists.keySet();
  }
  
  /**
  *@param name nimi
  *@return palautetaan kontaktilista nimen perusteella.
  */
  public ContactList getContactList(String name){
    return (ContactList) this.contactlists.get(name);
  }
  
  /**
  *Listn kontaktitieto.
  *
  *@param contact listtv kontakti.
  */
  public void addContact(Contact contact){
    this.contacts.put(contact.getName(), contact);
  }
  
  /**
  *Listn alikontaktilista.
  *
  *@param clist listtv alikontaktilista
  */
  public void addContactList(ContactList clist){
    this.contactlists.put(clist.getName(), clist);
  }
  
  
  /**
  *Poistetaan kontaktitieto.
  *
  *@param name poistettavan kontaktitiedon nimi
  */
  public void removeContact(String name){
    this.contacts.remove(name);
  }
  
  /**
  *Poistetaan alikontaktilista.
  *
  *@param name poistettavan alikontaktilistan nimi
  */
  public void removeContactList(String name){
    this.contactlists.remove(name);
  }
  
  /**
  *Luodaan XML.
  */
  public void createXML( Node par ) {
    Document dom = par.getOwnerDocument();
    Element el = dom.createElement("contactlist");
    el.setAttribute("name", this.name);
    par.appendChild(el);
    Iterator li = this.contactlists.keySet().iterator();
    while (li.hasNext()) {
      String key = (String)li.next();
      ((ContactList)this.contactlists.get(key)).createXML(el);
    }
    Iterator ci = this.contacts.keySet().iterator();
    while (ci.hasNext()) {
      String key = (String)ci.next();
      ((Contact)this.contacts.get(key)).createXML(el);
    }
  
  }
  
  
  /**
  *Ladataan XML.
  */
  public boolean loadXML( Node cont ) {
    Element root = (Element) cont;
    if (!root.getTagName().equals("contactlist"))
      return false;
    this.name = root.getAttribute("name");
    Element el = (Element)root.getFirstChild();
    while (el != null) {
      ContactList l = new ContactList();
      Contact c = new Contact();
      if (c.loadXML(el)) {
        this.addContact(c);
      }
      else if (l.loadXML(el)) {
        this.addContactList(l);
      }
      el = (Element)el.getNextSibling();
    }
    return true;
  }
  
  
  /**
  *Testausta varten
  */
  public static void main (String[] args) {
    try {
    ContactList l = new ContactList("A_CONTACT_LIST");
    Contact c = new Contact("ConvergeGroup");
    c.set("email", "converge-list@cs.helsinki.fi");
    l.addContact(c);
    ContactList l2 = new ContactList("Inner");
    l2.addContact(new Contact("A Contact"));
    l.addContactList(l2);
    Document dom = XMLUtil.newXMLDocument();
    l.createXML(dom);
    TextWriter tw = new TextWriter(dom);
    tw.write(System.out);
    System.out.println("");
    ContactList l3 = new ContactList();
    l3.loadXML(dom.getDocumentElement());
    dom = XMLUtil.newXMLDocument();
    l3.createXML(dom);
    //tw = new TextWriter(dom);
    //tw.write(System.out);
    XMLUtil.outputPrinting(dom, null);
    System.out.println("");
  }catch(Exception e){e.printStackTrace();}
  
  }
  
}
