/*
Tiedosto: Profile.java
Luokat:   Profile

Tekij:   Olli Pettay
Luonti:   17.11.2002

Muutokset:
  17.11.2002, Olli Pettay, luokan luonti
  19.11.2002, Olli Pettay, metodien mrittelyj
  05.12.2002, Olli Pettay, XML-muodostus ja luku.

  
  TODO: Koodin kommentointi
*/


package converge.kernel;

import org.w3c.dom.*;
import java.util.*;
import org.apache.xindice.xml.TextWriter;
/**
*Profile-luokka kuvaa kyttjn/ryhmn yht profiilia.
*/
public class Profile implements XMLContentInterface{
  
  public static final String DEFAULT = "DEFAULT";
  public User user;
  private String name;
  private boolean active;
  private LinkedHashMap rules;
  private HashMap contextModelValues;
  private HashMap scheduledRules;
  private boolean dirty;
  
  private Profile(){}
  
  protected Profile(User user){
    this.user = user;
    this.scheduledRules = new HashMap();
    this.rules = new LinkedHashMap();
    this.contextModelValues = new HashMap();
  }
  
  public Profile(User user, String name) {
    this.scheduledRules = new HashMap();
    this.rules = new LinkedHashMap();
    this.contextModelValues = new HashMap();
    this.name = name;
    this.user = user;
    this.dirty = false;
    this.active = false;
  }
  
  /**
  *Tll hetkell DEFAULT-niminen profiili on oletusprofiili
  */
  public boolean isDefaultProfile(){
    return this.name.equals(DEFAULT);
  }
  
  /**
  *Profiilin nimi
  */
  public String getName() {
    return this.name;
  }
   
  //Tm ei ole kytss - viel
  public boolean isDirty() {
    return this.dirty;
  }
  
  /**
  *Onko profiili aktiivinen
  */
  public synchronized boolean isActive() {
    return this.active;
  }
  
  /**
  *Aseta profiili aktiiviseksi tai passiiviseksi
  *
  *@param active jos true, niin profiili aktivoituu, jos false passivoituu
  */
  public synchronized void setActive(boolean active) {
    this.active = active;
  }
  
  
  /**
  *Snnn lisminen profiiliin.
  *
  *@param rule snt
  */
  public void setRule(Rule rule) {
    this.rules.put(rule.getName(), rule);
    this.dirty = true;
  }
  
  /**
  *Palautetaan snt nimen perusteella.
  *
  *@param name halutun snnn nimi.
  */
  public Rule getRule(String name) {
    return (Rule)this.rules.get(name);
  }
  
  
  /**
  *Poistetaan snt nimen perusteella.
  *
  *@param name poistettavan snnn nimi
  */
  public void deleteRule(String name) {
    this.rules.remove(name);
    this.dirty = true;
  }
  
  
  /**
  *Sntjen nimet
  */
  public String[] getRuleNames() {
    Set keys = this.rules.keySet();
    return (String[])keys.toArray(new String[0]);
  }
  
  
  public Rule[] getRules() {
      return (Rule[])this.rules.values().toArray(new Rule[this.rules.size()]);
  }
  
  /**
  *Tm on tarpeen vasta sitten kun 'dirty'-ksittely toimii kaikkialla.
  */
  public void save(){
    if (this.dirty)
      this.user.save();
    this.dirty = false;
  }
  
  
  /**
  *Ajastuksen / ajastetun snnn lisminen
  *
  *@param rule ajastin, joka pit sislln sntj
  */
  public void addScheduledRule(Scheduler rule) {
    //System.out.println("Scheduled rule added:" + rule.getName());
    this.scheduledRules.put(rule.getName(), rule);
    this.dirty = true;
  }
  
  
  /**
  *Palautetaan ajastin nimen perusteella.
  *
  *@param name ajastimen nimi
  */
  public Scheduler getScheduledRule(String name) {
    return (Scheduler) this.scheduledRules.get(name);
  }
  
  
  /**
  *Poistetaan ajastin profiilista.
  *
  *@param name ajastimen nimi
  */
  public void deleteScheduledRule(String name) {
    this.scheduledRules.remove(name);
    this.dirty = true;
    return;
  }
  
  /**
  *Palautetaan kaikki profiiliin liitetyt ajastimet taulukossa.
  */
  public Scheduler[] getScheduledRules() {
    return (Scheduler[])this.scheduledRules.values().toArray(new Scheduler[this.scheduledRules.size()]);
  }
  
  
  public synchronized void createXML(Node parent) {
     Document dom = (Document) parent;
     Element root = dom.createElement("profile");
     root.setAttribute("name", this.name);
     dom.appendChild(root);
     Element el = dom.createElement("rule-set");
     root.appendChild(el);
     Set rs = this.rules.keySet();
     Iterator it = rs.iterator();
     while (it.hasNext()) {
      Rule r = (Rule) this.rules.get(it.next());
      r.createXML(el);
     }
     
     el =dom.createElement("scheduledrules");
     root.appendChild(el);
     it = this.scheduledRules.keySet().iterator();
     while (it.hasNext()) {
      Scheduler s = this.getScheduledRule((String) it.next());
      s.createXML(el);
     }
     
     el = dom.createElement("contextmodelvalues");
     root.appendChild(el);
     rs = this.contextModelValues.keySet();
     it = rs.iterator();
     while (it.hasNext()) {
      String name = (String) it.next();
      Integer r = (Integer) this.contextModelValues.get(name);
      Element el2 = dom.createElement("contextmodelvalue");
      el2.setAttribute("name", name);
      el2.setAttribute("value", r.toString());
      el.appendChild(el2);
     }
  }
  
  
  public synchronized boolean loadXML(Node cont){
    Element content = (Element)cont;
    Document dom = cont.getOwnerDocument();
    if (!content.getTagName().equals("profile"))
      return false;
    this.dirty = false;
    this.name = content.getAttribute("name");
    Element child = (Element) content.getFirstChild();
    while (child != null) {
      if (child.getTagName().equals("rule-set")) {
        Element subchild = (Element)child.getFirstChild();
        while (subchild != null) {
          Rule r = new Rule();
          r.loadXML(subchild);
          this.setRule(r);
          subchild = (Element) subchild.getNextSibling();
        }
      }
      else if (child.getTagName().equals("scheduledrules")) {
        Element subchild = (Element)child.getFirstChild();
        while (subchild != null) {
          Scheduler r = new Scheduler();
          r.loadXML(subchild);
          this.addScheduledRule(r);
          subchild = (Element) subchild.getNextSibling();
        }
      }
      else if (child.getTagName().equals("contextmodelvalues")) {
        Element subchild = (Element)child.getFirstChild();
        while (subchild != null) {
          this.setContextModelValue(subchild.getAttribute("name"), 
                  new Integer(subchild.getAttribute("value")).intValue());
          subchild = (Element) subchild.getNextSibling();
        }
      }
      child = (Element) child.getNextSibling();
    }
    
    return true;
  }
  
  
  
  /**
  *Asetetaan kontekstimallin kynnysarvo.
  *
  *@param name kontekstimallin nimi
  *@param value kynnysarvo 
  */
  public void setContextModelValue(String name, int value) {
    contextModelValues.put(name, new Integer(value));
    this.dirty = true;
  }
  
  
  /**
  *Palautetaan kontekstimallin kynnysarvo.
  *Jos arvoa ei ole asetettu, palautetaan 101.
  *
  *@param name kontekstimallin nimi
  */
  public int getContextModelValue(String name) {
    Integer val = (Integer)this.contextModelValues.get(name);
    if (val != null) return ((Integer)val).intValue();
    return 101;
  }
  
  /**
  *Poistetaan profiilista viittaus kontekstimalliin.
  */
  public void removeContextModelValue(String name){
    this.contextModelValues.remove(name);
    this.dirty = true;
  }
  
  
  /**
  *Palautetaan profiiliin liitettyjen kontekstimallien nimet.
  */
  public String[] getContextModels() {
    return (String[])this.contextModelValues.keySet().toArray(new String[0]);
  }
  
  /**
  *Tulostetaan oletusprofiili System.out -virtaan
  *
  */
  public static void main(String[] args) {
    Profile prof = new Profile(new User("", ""), DEFAULT);
    Rule rule = new Rule("All_to_INBOX");
    rule.consequence(Rule.EVENT_ADD, "ToView", "INBOX");
    prof.setRule(rule);
    Document dom = XMLUtil.newXMLDocument();
    prof.createXML(dom);
    try {
    new TextWriter(dom).write(System.out);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}




