/*
Tiedosto: ProfileHandler.java
Luokat:   ProfileHandler

Tekij:   Olli Pettay
Luonti:   17.11.2002


Muutokset:
  17.11.2002 Olli Pettay  
             Luokan luonti
  10.12.2002 Mikko Hiipakka
             Luokan toiminta rakennettu
  15.12.2002 Mikko Hiipakka
             Rakenne tarkistettu
*/


package converge.kernel;

import java.util.HashMap;
import org.w3c.dom.Document;
import converge.service.Message;
import converge.attribute.MessageAttributeInterface;
/**
* Luokka tarjoaa viestiolion mukana tulevaan kyttjn liitettyjen profiilien ksittelyn.
*/
public class ProfileHandler {
  
  private Message msg = null;
  //Ryhmien oletusprofiilien silytys kunnes tulee kyttj
  //kohtainen ksittely, jossa kytettn samaa eventti pohjana
  private static HashMap baseEvents = new HashMap();
  private static HashMap baseCounter = new HashMap();
  
  /**
   * Luokan konstruktori, jolla asetetaan ilmentym ksittely valmiiseen tilaan
   * @param msg Viestiolio, jonka mukana tulevan kyttjolion profiileja ksitelln 
  */
  public ProfileHandler(Message msg) {
    this.msg = msg;
  }

  
  /**
  * Kynnist 
  */
  public void startProcessing() {
    if (this.msg == null)
      return;
    RuleHandler rHandler = new RuleHandler();
    EventMonitor eMonitor = EventMonitor.getInstance();
    User currentUser = this.msg.getUser();
    boolean currIsGroup = ( currentUser instanceof Group );
    //Eventti lytyy, jos ksittely tapahtuu ryhmn jsenelle henk.koht.
    //jolloin ryhmn ksittely on alustanut jo eventin
    ActionEvent base_event = getBaseEvent( this.msg.getMessageID() );
    
    Profile base_prof = null;
    
    Profile[] currentProfiles = currentUser.getProfiles();
    
    //oletusprofiili
    base_prof = currentUser.getDefaultProfile();
    rHandler.processRules( this.msg, base_prof.getRules(), base_event);

    
    for( int i=0; i<currentProfiles.length; i++ ){
      Profile prof = currentProfiles[i];

      //Ei ksitell passiivisia profiileja tai oletusprofiilia uudestaa 
      if ( !prof.isActive() || prof.isDefaultProfile() )
        continue;

      //Tarkistetaan aktiivisen profiilin asettamat kontekstimallien kynnysarvot
      if (this.limitCheck(prof)) {
        //Kaikki profiilit ksittelevt samaa ActionEventti 
        //(kertn arvoja, joita muut profiilit voivat halutessaan muuttaa)
        rHandler.processRules( this.msg, prof.getRules(), base_event );
      }
    }
    
    if ( currIsGroup ){
      UserNameCollection unC = ((Group)currentUser).getUserNames();
      String[] memberColl = unC.getUserNames();
      //Ryhmn tapahtumia ei lhetet eteenpin vaan tallennetaan 
      //jokaisen ryhmn kuuluvan kyttjn henk.koht. ksitely varten
      addBaseEvent( this.msg.getMessageID(), base_event, memberColl.length);
      //Tehdn viesti-oliosta kopio (thn htn XML:n kautta)
      Document dom = XMLUtil.newXMLDocument();
      this.msg.createXML(dom);
      
      UserManager um = UserManager.getInstance();
      for (int i=0; i<memberColl.length; i++){
        //Ladataan ryhmn jsen
        User member = um.getUser( memberColl[i] );
        //Viestin kopio
        Message msgCopy = new Message( member, dom );
        //Ryhmnjsenen henk.koht. ksittelyn aloitus
        ContextModelHandler.incomingMessage( msgCopy );
      }
    }else{
        //Jos oli User-olio lhetetn eteenpin ksittelyyn
        eMonitor.incomingEvent(msg, base_event);
    }
  }  
  
  
//***************************************************************************************
//***************************************************************************************
//                              PRIVATE METODIT
//***************************************************************************************
  
  
  //Profiilin sisltmien kontekstimallien kynnysarvojen tutkiminen 
  //verrattuna viesti-olion ContextModelValueAttribute-arvoja vastaan
  private boolean limitCheck(Profile profile) {
    String[] cm = profile.getContextModels();
    //Jos yhden kontekstimallin arvo ylitt kyseiselle mallille asetetun
    //kynnysarvon ---> true
    for ( int i=0; i<cm.length; i++ ) {
      String currentModel = cm[i];
      MessageAttributeInterface attr = this.msg.getMessageAttribute(currentModel);
      if ( attr != null ){
        int value = ((Integer)attr.getValue()).intValue(); 
        if (  value >= profile.getContextModelValue(currentModel))
          return true;
      }
    }
    return false;
  }

  
  private static synchronized ActionEvent getBaseEvent( String key ){
    ActionEvent ae = (ActionEvent)baseEvents.get( key );
    //Palautettaan uusi ActionEvent, jos ei lytynyt
    if (ae == null ) {
      return new ActionEvent();
    }
    else{
      //Vhennetn laskuria
      int count = ((Integer)baseCounter.remove( key )).intValue();
      count--;
      if (count > 0 )
        baseCounter.put( key, new Integer(count) );
      else{
        baseEvents.remove( key );
      }
    }
    return new ActionEvent(ae);
  }
  
  private static synchronized void addBaseEvent( String key, ActionEvent ae, int count ){
    baseEvents.put( key, ae );                 //Eventti talteen
    baseCounter.put( key, new Integer(count)); //Laskuri Eventille
  }
}
