/*

Tiedosto   : ScheduleManager.java
Luokat     : ScheduleManager

         
Tekij     : Olli Pettay
Luonti pvm : 25.11.2002


Muutokset :

25.11.2002 Olli Pettay
           Luotu

*/


package converge.kernel;

import java.util.*;
import java.io.*;
import org.w3c.dom.*;


/**
*Kyttjien profiileissa olevien ajastusten hallinta.
*/
public class ScheduleManager extends Thread{ 

  public static final int sleepTime = 31; //joka n. sekunti
  private static ScheduleManager sm;
  private boolean stop = false;
  private boolean stopped = false;
  
  public synchronized static ScheduleManager getInstance() {
    if (sm == null) {
      sm = new ScheduleManager();
      sm.start();
    }
    return sm;
  }
  
  private synchronized boolean shouldStop(){
    return this.stop;
  }
  
  
  /**
  *ScheduleManagerin sulkeminen
  */
  public synchronized void shutdown(){
    this.stop = true;
    while(this.stopped != true) {
      try {
        wait();
      }
      catch (Exception e) {Log.stack(e);}
    }
  }
  

  public void run() {
    int sleeprounds = 10;
    UserManager um = UserManager.getInstance();
    //DEBUG
    System.out.println("ScheduleManager running...");
    //DEBUG
    //Kydn lpi jokaisen kyttjn aktiivisten profiilien
    //ajastetut snnt, ja mikli snt tulee laukaista,
    //Annetaan DroolsHandlerille uuden tyhjn ActionEventin kanssa.
    //Tmn jlkeen EventMonitorille
    while (!this.shouldStop()) {
      try {
       //Tm voisi tehd mys interrupt-metodilla, mutta tmkin toiminee.
        for (int i = 0; i < sleeprounds; ++i) {
          if (this.shouldStop())
            break;
          Thread.sleep(ScheduleManager.sleepTime * 100);
        }
        if (this.shouldStop())
            break;
      }catch (InterruptedException e) {
        break;
      }
      
      long time = System.currentTimeMillis();
      
      //Ryhmien liittminen sotki hieman koodia,
      String[] usernames = um.getUserNames();
      String[] groupnames = um.getGroupNames();
      for (int u = 0; u < 2; ++u) {
        String[] users;
        if (u == 0)
          users = usernames;
        else 
          users = groupnames;

        for (int i = 0; i < users.length; ++i) {
          User user;
          if (u == 0)
            user = um.getUser(users[i]);
          else 
            user = um.getGroup(users[i]);
          if (user != null && user.readLockNoWait(this)) {
            if (user instanceof Group)
              Log.output("schedulemanager").
                        println("ScheduleManager is processing group: " 
                                                           + user.getName());
            else 
              Log.output("schedulemanager").
                        println("ScheduleManager is processing user: " 
                                                           + user.getName());
            Profile[] profiles = user.getProfiles();
            for (int j = 0; j < profiles.length; ++j) {
              if (profiles[j].isActive()) {
                Scheduler[] ss = profiles[j].getScheduledRules();
                for (int k = 0; k < ss.length; ++k) {
                  if ((ss[k].exact() && (ss[k].getLastTime() < time)) ||
                     ((!ss[k].exact()) && 
                    ((ss[k].getInterval() + ss[k].getLastTime()) < time))) {
                    ScheduledRuleHandler srh = 
                            new ScheduledRuleHandler(user, ss[k].getRules());
                    srh.start();
                    if (ss[k].exact())
                      ss[k].setLastTime(Long.MAX_VALUE);
                    else
                      ss[k].setLastTime(time);
                  } 
                  
                }
              }
            }
            user.releaseLock(this);
          }
        }
      }
    }
    this.stopped = true;
    //notifyAll();
  }
  
}


/**
*Mikli ScheduleManager huomaa, ett jokin ajastus
pit kynnist, luodaan uusi ScheduledRuleHandler-sie, joka
huolehtii ajastuksen toiminnallisuudesta.
*
*/
class ScheduledRuleHandler extends Thread {
  private Rule[] rules;
  private User user;
  private ActionEvent event;
  
  public ScheduledRuleHandler(User user, Rule[] rules) {
    this.user = user;
    this.rules = rules;
    this.event = new ActionEvent();
  }
  
  
  public void run() {
    if (this.user != null && this.user.readLock(this)) {
      System.out.print("ScheduleHandler running for " + this.user.getName() + "...");
      RuleHandler rh = new RuleHandler();
      rh.processRules(this.user, this.rules, this.event);
      EventMonitor.getInstance().incomingEvent(user, event);
      System.out.println("done.");
      this.user.releaseLock(this);
    }
  }

}
