/*
Tiedosto: User.java
Luokat:   User

Tekij:   Olli Pettay
Luonti:   17.11.2002

Muutokset:
17.11.2002 Olli Pettay
           Luokan luonti

19.11.2002 Olli Pettay, 
           Datasourcet listty

19.11.2002 Mikko Hiipakka
           ArrayList contextModel listty - tarkoitettu kyttjn 
           kontekstimallien hallinnointiin
           public ArrayList getContextModel - aksessori listty 
           public void addContextModel( ContextModel value ) - listty
      
20.11.2002 Olli Pettay
            ContextModel -hallinnointi kopioitu mys
            profiilien hallinnointiin.
            Dirty -kentn lisys.
            ContactList -listty osittain, mutta toteutetaanko
            ArrayList ei ole kovin hyv vaihtoehto contekstimalleihin,
              eik profiileihin. Kenties Map olisi parempi?
8.12.2002  Olli Pettay
            Lukkojen testaus yms.
            

*/


package converge.kernel;
import java.util.*;
import converge.service.DataSource;
import converge.attribute.*;

/**
*Jrjestelmn kyttjolio.
*
*/
public class User {

  private HashMap contextModels;
  private HashMap profiles;
  private ContactList contacts;
  private HashMap datasources;
  private HashMap userProperties;
  
  //Onko kyttjn tehty muutoksia
  private boolean dirty = false;
  
  
  private String name;
  private String password;
  
  private Set readLocks;
  private Object theWriteLock;
  private boolean waitForWriteLock;
  private Profile defaultProfile = null;
  private boolean deleted = false;

/**
* Luodaan uusi kyttj nimen ja salasanan perusteella.
*/
  protected User(String name, String password) {
    this.datasources = new HashMap();
    this.readLocks = Collections.synchronizedSet(new HashSet());
    this.contacts = new ContactList("This is " + name + "'s contactlist.");
    this.profiles = new HashMap();
    this.userProperties = new HashMap();
    this.theWriteLock = null;
    this.waitForWriteLock = false;
    this.contextModels = new HashMap();
    this.name = name;
    this.password = password;
    this.dirty = true;
  }
  
////////////////////////////////////////////////////////////////////
////////////////// Locks ///////////////////////////////////////////

  //Tuleeko jokaisen metodin tutkia onko kysyvll oliolla
  //luku- tai kirjoitusoikeus? EI
  
  public synchronized boolean readLock(Object locker) {
    if (this.hasWriteLock(locker))
      return false;
    if (this.hasReadLock(locker))
      return true;
    while (this.isWriteLocked() || this.waitForWriteLock) {
      try {
        this.wait();
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    if (this.deleted)
      return false;
    this.readLocks.add(locker);
    Log.output("User").println("User " + this.getName() + " readLocked by          " + locker.toString());
    return true;
  }
  
  
  public synchronized boolean readLockNoWait(Object locker) {
    if (this.hasWriteLock(locker))
      return false;
    if (this.hasReadLock(locker))
      return true;
    if (this.isWriteLocked() || this.waitForWriteLock) {
      return false;
    }    
    if (this.deleted)
      return false;
    this.readLocks.add(locker);
    Log.output("User").println("User " + this.getName() + " readLocked(no wait) by " + locker.toString());
    return true;
  }
  
  
  public synchronized boolean writeLock(Object locker){
    if (this.hasReadLock(locker))
      return false;
    if (this.hasWriteLock(locker))
      return true;
    Log.output("User").println("User " + this.getName() + " trying to writeLock by " + locker.toString());
    while (this.theWriteLock != null || this.waitForWriteLock)
    {
      try {
        this.wait();
      }
      catch (Exception e) {
        Log.stack(e);
      }
    }
    if (this.deleted)
      return false;
    this.waitForWriteLock = true;
    Log.output("User").println("User " + this.getName() + " waiting for writeLock: " + locker.toString());
    while(!this.readLocks.isEmpty()) {
      try {
        this.wait();
      }
      catch (Exception e) {
        Log.stack(e);
      }
    }
    if (this.deleted)
      return false;
    this.waitForWriteLock = false;
    this.theWriteLock = locker;
    Log.output("User").println("Has readlocks? " + (!this.readLocks.isEmpty()));
    Log.output("User").println("User " + this.getName() + " writeLocked by         " + locker.toString());
    return true;
  }

  public synchronized boolean releaseLock(Object locker){
    if (locker == this.theWriteLock) {
      this.theWriteLock = null;
      if (!this.deleted)
        try {
          this.save();
        } catch (Exception e) {
          Log.stack(e);
        }
    } 
    this.readLocks.remove(locker);
    boolean re = !(this.hasLock(locker));
    this.notifyAll();
    Log.output("User").println("User " + this.getName() + " released by            " + locker.toString());
    return re;
  }
  
  public synchronized boolean hasLock(Object ob) {
    return (this.hasReadLock(ob) || this.hasWriteLock(ob));
  }
  
  public synchronized boolean hasReadLock(Object ob) {
    return this.readLocks.contains(ob);
  }
  
  public synchronized boolean hasWriteLock(Object ob) {
    return (ob == this.theWriteLock);
  }
  public synchronized boolean isWriteLocked() {
    return (this.theWriteLock != null);
  }
  
////////////////////////////////////////////////////////////////////
////////////////// 'normal ' methods////////////////////////////////


/**
* Merkitn kyttj tuhotuksi.
*/
public void setDeleted() {
  this.deleted = true;
}
  
/**
*Onko kyttj poistettu jrjestelmst.
*/
public boolean isDeleted() {
  return this.deleted;
}
/**
* Palautetaan kyttjn nimi.
*/
 public String getName(){
  return this.name;
 }

 
 /**
 *Palautetaan kyttjn salasana.
 */
 protected String getPassword() {
  return this.password;
 }
 
  /**
   * Kyttj-olion sisltmien kontekstimallien palauttava aksessori
   * @return ContextModel[], joka sislt viitteet kyttjn konteksitmalleihin
  */
  public ContextModel[] getContextModels(){
    return (ContextModel[])this.contextModels.values().toArray(new ContextModel[this.contextModels.size()]);
  }


  /**
   * Kontekstimallin liittminen kyttjn
   *
   * @param value Uusi kontekstimalli liitettvksi kyttjn
  */
  public void addContextModel( ContextModel value ){
    //Kyttjn ei voi olla liitettyn kontekstimallia, jonka nimi on jokin attribuutin looginen nimi
    //Ongelma tulisi, kun viestioliolta pyydetn ContextModelValueAttribute-attribuuttia profiilien
    //kynnysarvo vertailua varten.
    MessageAttributeInterface attr = AttributeLoader.newMessageAttributeInstance( value.getModelName() );
    if (attr == null ){
      this.contextModels.put(value.getModelName(), value );
      this.dirty = true;
    }
  }

  /**
  *Poistetaan kontekstimalli
  *
  *@param name poistettavan kontekstimallin nimi
  */
  public void deleteContextModel(String name) {
    this.contextModels.remove(name);
    Profile[] p = this.getProfiles();
    for (int i = 0; i < p.length; ++i)
      p[i].removeContextModelValue(name);
  }
  
  
  /**
  *Palautetaan kontekstimalli nimen perusteella.
  *
  *@param name kontekstimallin nimi.
  */
  public ContextModel getContextModel(String name) {
    return (ContextModel) this.contextModels.get(name);
  }
  
  /**
   * Kyttj-olion sisltmien profiilien palauttava aksessori
   * @return Profile[], joka sislt viitteet profiileihin
  */
  
  public Profile[] getProfiles(){
    return (Profile[])this.profiles.values().toArray(new Profile[this.profiles.size()]);
  }


  /**
   * Profiilin liittminen kyttjn
   * @param value uusi profiili liittetvksi kyttjn
  */
  public void addProfile( Profile value ){
    this.profiles.put(value.getName(),  value );
    this.dirty = true;
  }

  /**
  * Profiilin poistaminen.
  *
  *@param prof poistettava profiili.
  */
  public void deleteProfile(Profile prof) {
    if (prof.isDefaultProfile())
      return;
    this.profiles.remove(prof.getName());
  }
  
  /**
  *Palautetaan kyttjn oletusprofiili.
  *
  */
  public Profile getDefaultProfile() {
    return this.getProfile(Profile.DEFAULT);
  }
  
  /**
  * Profiili nimen perusteella.
  */
  public Profile getProfile(String name) {
    return (Profile) this.profiles.get(name);
  }
  
  
  
  /**
  *Tiedonlhteen lisminen.
  *
  *@param src liitettv tiedonlhde
  */
  public void addDataSource(DataSource src) {
    this.datasources.put(src.getName(), src);
    System.err.println("DATASOURCE: " + src.getName());
    //this.dirty = true;
  }
  
  
  /**
  * Poistetaan tiedonlhden
  *
  *@param name poistettavan tiedonlhteen nimi.
  */
  public void deleteDataSource(String name) {
    this.datasources.remove(name);
  }
  
  /**
  *Palautetaan tiedonlhde nimen perusteella.
  *
  */
  public DataSource getDataSource(String name) {
    DataSource ds = (DataSource) this.datasources.get(name);
    return ds;
  }
  
  
  /**
  *Kyttjn kaikkien tiedonlhteiden nimet taulukossa.
  *
  */
  public String[] getDataSourceNames(){
    return (String[]) datasources.keySet().toArray(new String[datasources.size()]);
  }
  
  

  
  /**
  *Kontaktilista.
  *
  */
  public ContactList getContactList() {
    return this.contacts;
  }

  
  //Ei kytss - viel
  protected void setDirty() {
    this.dirty = dirty;
  }
  
  //Ei kytss
  protected boolean isDirty() {
    return this.dirty;
  }
  
  
  /**
  * Kyttjn ominaisuuksien kysely.
  * 
  *@return palautetaan UserPropertyInterfacen toteuttava olio tai,
  * mikli jrjestelm ei tue ominaisuutta, null.
  */
  public UserPropertyInterface getProperty(String name){
    if (this.userProperties.containsKey(name))
      return (UserPropertyInterface) this.userProperties.get(name);
    UserPropertyInterface u = AttributeLoader.newUserPropertyInstance(name);
    if ( u == null ){
      Log.output("UserProperty").println("UserPropertyn liittminen eponnistui: " + name);
                return null;
      }
    u.setName(name);
    u.setUser(this);
    this.userProperties.put(name, u);
    return u;
  }
  
  
  /**
  *Tallennus.
  *Tt ei tarvitse koskaan kutsua, mikli kirjoituslukkoja on kytetty
  *oikein.
  */
  public void save() {
    UserManager.getInstance().saveUser(this);
  }
  
}

