/*
Tiedosto: UserManager.java
Luokat:   UserManager

Tekij:   Olli Pettay
Luonti:   17.11.2002

Muutokset:
  17.11.2002  Luokan luonti

*/


package converge.kernel;
import java.util.*;
import org.apache.xindice.server.*;
import org.apache.xindice.core.*;
import org.apache.xindice.core.data.*;
import org.apache.xindice.server.services.*;
import org.apache.xindice.util.*;
import org.apache.xindice.xml.dom.DOMParser;
import org.w3c.dom.*;
import java.io.*;
import converge.service.DataSource;

/**
*UserManager hallinnoi jrjestelmn kyttji ja ryhmi.
*/
public class UserManager {
  private static String masterPassword = null;
  private static UserManager um = null;
  private Map users;
  private Map groups;
  private DatabaseProvider db;
  
  
  private UserManager() {
    //initialization here
    users = Collections.synchronizedMap(new HashMap());
    groups = Collections.synchronizedMap(new HashMap());
    db = DatabaseProvider.getInstance();
    this.loadUsers();
    this.loadGroups();
    //Load everything from db
  }
  
  /**
   * Palauttaa jrjestelmn ainoan UserManager-ilmentymn
  */ 
  public synchronized static UserManager getInstance() {
    if (um == null)
      um = new UserManager();
    return um;
  }


  /**
   * UserManager-ilmentymn sulkeminen
  */   
  public synchronized void shutdown() {
    saveAllUsers(10);
  }
  
  /**
  *Uuden ryhmn luominen.
  */
  public synchronized Group newGroup(String name, String password) {
    if (name == null || this.hasGroup(name) || password == null) 
      return null;
    Group aUser = new Group(name, Crypt.encode(password, password.getBytes()));
    try {
      Document dom = DOMParser.toDocument(new FileInputStream(new File("config/groupprofile.xml")));
      Profile prof = new Profile(aUser);
      prof.loadXML(dom.getDocumentElement());
      aUser.addProfile(prof);
      prof.setActive(true);
    } catch(Exception e) {
      Log.stack(e);
    }
    this.saveUser(aUser);
    this.groups.put(name, aUser);
    return aUser;
  }
  
  /**
  *Uuden kyttjn luominen.
  *
  */
  public synchronized User newUser(String name, String password){
    if (name == null || this.hasUser(name) || password == null) 
      return null;
    User aUser = new User(name, Crypt.encode(password, password.getBytes()));
    try {
      Document dom = DOMParser.toDocument(new FileInputStream(new File("config/profile.xml")));
      Profile prof = new Profile(aUser);
      prof.loadXML(dom.getDocumentElement());
      aUser.addProfile(prof);
      prof.setActive(true);
    } catch(Exception e) {
      Log.stack(e);
    }
    this.saveUser(aUser);
    this.users.put(name, aUser);
    return aUser;
  }
  
  /**
  *Palautetaan jrjestelmss oleva ryhm nimen perusteella.
  */
  public synchronized Group getGroup(String name) {
    return (Group) this.groups.get(name);
  }
  
  /**
  *Palautetaan jrjestelmss oleva kyttj nimen perusteella.
  */
  public  synchronized User getUser(String name) {
    return (User) this.users.get(name);
  }
  
  /**
  *Palautetaan jrjestelmss oleva ryhm nimen ja salasanan perusteella.
  */
  public synchronized Group getGroup(String name, String password) {
    if (!this.hasGroup(name)) return null;
    Group aUser = (Group) this.groups.get(name);
    String dec = Crypt.decode(password, aUser.getPassword());
    if (dec != null && dec.equals(password))
      return aUser;
    return null;
  }
  
  
  /**
  *Palautetaan jrjestelmss oleva kyttj nimen ja salasanan perusteella.
  */
  public synchronized User getUser(String name, String password) {
    if (!this.hasUser(name)  || password == null) 
      return null;
    User aUser = (User) this.users.get(name);
    if (aUser == null) 
      return null;
    String dec = Crypt.decode(password, aUser.getPassword());
    if (dec != null && dec.equals(password))
      return aUser;
    return null;
  }
  
  /**
  *Onko jrjestelmsss ryhm.
  *
  *@param name ryhmn nimi
  */
  public synchronized boolean hasGroup(String name) {
    if (name == null)
      return false;
    return this.groups.containsKey(name);
  }
  
  
  /**
  *Onko jrjestelmss kyttj
  *
  *@param name kyttjn nimi
  */
  public synchronized boolean hasUser(String name) {
    if (name == null)
      return false;
    return this.users.containsKey(name);
  }
  
  /**
  *Poistetaan ryhm.
  */
  public synchronized void deleteGroup(String name) {
    if (!this.hasGroup(name))
      return;
    this.groups.remove(name);
    this.db.removeGroupCollection(name);
  }
  
  
  /**
  *Poistetaan kyttj
  */
  public synchronized void deleteUser(String name){
    if (!this.hasUser(name))
      return;
    User user = this.getUser(name);
    user.writeLock(this);
    user.setDeleted();
    this.users.remove(name);
    this.db.removeUserCollection(name);
    String[] groups = this.getGroupNames();
    for (int i = 0; i < groups.length; ++i) {
      Group group = this.getGroup(groups[i]);
      group.writeLock(this);
      group.getUserNames().removeUser(name);
      group.releaseLock(this);
    }
    user.releaseLock(this);
  }
  
  /**
  *Kyttjn tallennus
  */
  public synchronized void saveUser(User user) {
    //Seuraava toiminnallisuus voidaan laittaa plle vasta,
    //kun dirty-toimii kaikilla kyttjn oliolla, esim. Rule, ContextModel jne.
    //if (user.isDirty()) this.saveUserNow(user);
    this.saveUserNow(user);
  }
  
  /**
  *Kyttjn tallennus.
  */
  private void saveUserNow(User user) {
    if (user == null)
      return;
    String name = user.getName();
    org.apache.xindice.core.Collection userCol = null;
    if (!(user instanceof Group)) {
      userCol = this.db.getUserCollection(name);
      if (userCol == null)
        userCol = this.db.newUserCollection(name);
      if (userCol == null) {//Tm on virhe!!!!
        return;
      }
    }
    else {
      userCol = this.db.getGroupCollection(name);
      if (userCol == null)
        userCol = this.db.newGroupCollection(name);
      if (userCol == null) { //Tm on virhe!!!!
        return;
      }
    }
    Document dom = XMLUtil.newXMLDocument();
    Element el = dom.createElement("user");
    el.setAttribute("name", user.getName());
    el.setAttribute("password", user.getPassword());
    user.getContactList().createXML(el);
    if (user instanceof Group)
      //UserNameCollection.createXML(el)
      ((Group)user).getUserNames().createXML(el);
    dom.appendChild(el);
    this.db.setDocument(userCol, DatabaseProvider.PREF, DatabaseProvider.PREFERENCE, dom);
    
    
    Set wasInDatabase = this.toSet(userCol, DatabaseProvider.DATASOURCES);
    String[] datas = user.getDataSourceNames();
    for (int i = 0; i < datas.length; ++i) {
      Document d = XMLUtil.newXMLDocument();
      DataSource ds = user.getDataSource(datas[i]);
      ds.createXML(d);
      this.db.setDocument(userCol, DatabaseProvider.DATASOURCES, datas[i], d);
      wasInDatabase.remove(datas[i]);
    }
    this.deleteDocuments(userCol, DatabaseProvider.DATASOURCES, wasInDatabase);
    
    
    
    wasInDatabase = this.toSet(userCol, DatabaseProvider.PROFILES);
    Profile[] profiles = user.getProfiles();
    for (int i = 0; i < profiles.length; ++i) {
      Document d = XMLUtil.newXMLDocument();
      profiles[i].createXML(d);
      this.db.setDocument(userCol, DatabaseProvider.PROFILES, profiles[i].getName(), d);
      wasInDatabase.remove(profiles[i].getName());
    }
    this.deleteDocuments(userCol, DatabaseProvider.PROFILES, wasInDatabase);
    
    
    
    wasInDatabase = this.toSet(userCol, DatabaseProvider.CONTEXTMODELS);
    ContextModel[] cms = user.getContextModels();
    for (int i = 0; i < cms.length; ++i) {
      Document d = XMLUtil.newXMLDocument();
      cms[i].createXML(d);
      this.db.setDocument(userCol, DatabaseProvider.CONTEXTMODELS, cms[i].getModelName(), d);
      wasInDatabase.remove(cms[i].getModelName());
    }
    this.deleteDocuments(userCol, DatabaseProvider.CONTEXTMODELS, wasInDatabase);
  }
  
  /**
  *Tallennetaan jrjestelmn kaikki kyttjt
  *
  */
  protected synchronized void saveAllUsers(int sleep){
    Set keys = this.users.keySet();
    Iterator it = keys.iterator();
    while(it.hasNext()) {
      this.saveUserNow((User)this.users.get(it.next()));
      try {
        Thread.sleep(sleep);
      } catch (Exception e) {
        Log.stack(e);
      }
    }
  }
  
  
  /**
  * Ladataan kyttjien tiedot tietokannasta.
  */
  protected synchronized void loadUsers(){
      try {
      Database db = this.db.getDatabase();
      org.apache.xindice.core.Collection usersCol = db.getCollection(DatabaseProvider.USERS);
      String[] userNames = usersCol.listCollections();
      for (int i = 0; i < userNames.length; ++i) {
        String name = userNames[i];
        User user = this.load(name, false);
        this.users.put(name, user);
      }
    }catch(Exception e) {
      Log.stack(e);
    }
  }
  
  
  /**
  *Ladataan ryhmien tiedot tietokannasta.
  */
  protected synchronized void loadGroups(){
    try {
      Database db = this.db.getDatabase();
      org.apache.xindice.core.Collection usersCol = db.getCollection(DatabaseProvider.GROUPS);
      String[] userNames = usersCol.listCollections();
      for (int i = 0; i < userNames.length; ++i) {
        String name = userNames[i];
        Group user = (Group)this.load(name, true);
        this.groups.put(name, user);
      }
    }catch(Exception e) {
      Log.stack(e);
    }
  }
  
  /**
  *Ladataan kyttjn tai ryhmn tiedot tietokannasta
  *
  *@param name nimi
  @param group onko kyseess ryhm.
  */
  private User load(String name, boolean group) {
    try {
    org.apache.xindice.core.Collection userCol;
    if (!group)
      userCol = this.db.getUserCollection(name);
    else
      userCol = this.db.getGroupCollection(name);
    Document dom = this.db.getDocument(userCol, DatabaseProvider.PREF, DatabaseProvider.PREFERENCE);
    Element el = dom.getDocumentElement();
    User user = null;
    if (!group)
      user = new User(name, el.getAttribute("password"));
    else
      user = new Group(name, el.getAttribute("password"));
    
    //Kontaktilista silytetn perferences.xml -tiedostossa
    Element child = (Element) el.getFirstChild();
    while (child != null) {
      if(!user.getContactList().loadXML(child))
        if (user instanceof Group)
          ((Group)user).getUserNames().loadXML(child);
       child = (Element) child.getNextSibling();
    }
    

    org.apache.xindice.core.Collection dataCol = userCol.getCollection(DatabaseProvider.DATASOURCES);
    String[] datas = dataCol.listDocuments();
    for (int i = 0; i < datas.length; ++i) {
      Document d = this.db.getDocument(dataCol, null, datas[i]);
      if (d != null && d.getDocumentElement() != null) {
        DataSource ds = DataSource.getDataSource(d.getDocumentElement().getTagName());
        ds.loadXML(d.getDocumentElement());
        user.addDataSource(ds);
      }
    }
    
    
    org.apache.xindice.core.Collection proCol = userCol.getCollection(DatabaseProvider.PROFILES);
    String[] profiles = proCol.listDocuments();
    for (int i = 0; i < profiles.length; ++i) {
      Profile p = new Profile(user);
      p.loadXML(this.db.getDocument(proCol, null, profiles[i]).getDocumentElement());
      user.addProfile(p);
      if (p.isDefaultProfile())
        p.setActive(true);
    }
    
    org.apache.xindice.core.Collection conCol = userCol.getCollection(DatabaseProvider.CONTEXTMODELS);
    String[] contextModels = conCol.listDocuments();
    for (int i = 0; i < contextModels.length; ++i) {
      ContextModel c = new ContextModel(this.db.getDocument(conCol, null, contextModels[i]).getDocumentElement());
      user.addContextModel(c);
    }
    
    return user;
    }catch (Exception e) {
      Log.stack(e);
    }
    return null;
  }
  
  /**
  *Jrjestelmn kyttjien nimet.
  *
  *
  */
  public synchronized String[] getUserNames(){
    return (String[])users.keySet().toArray(new String[0]);
  }
  
  /**
  *Jrjestelmn ryhmien nimet.
  *
  */
  public synchronized String[] getGroupNames(){
    return (String[])groups.keySet().toArray(new String[0]);
  }
  
  
  /**
  *Kertn kokoelman alikokoelman dokumenteihin viittaavat avaiment
  *Set-tietorakenteeseen.
  *
  */
  private Set toSet(org.apache.xindice.core.Collection collection, String sub) {
    Set set = new HashSet();
    String[] names = null;
    if ( collection != null && sub != null) {
      try {
      collection = collection.getCollection(sub);
      names = collection.listDocuments();
      }
      catch (Exception e) {
        Log.stack(e);
        return set;
      }
      if (names != null)
      for(int i = 0; i < names.length; ++i)
        set.add(names[i]);
    }
    return set;
  }
  
  /**
  *Poistetaan kokoelman alikokoelmasta dokumentit, avaimet
  *on saatu Set-tietorakenteessa.
  *
  */
  private void deleteDocuments(org.apache.xindice.core.Collection collection, String sub, Set set) {
    if (collection == null || sub == null || set == null || set.isEmpty())
      return;
    try {
      collection = collection.getCollection(sub);
    }
    catch (Exception e) {
        Log.stack(e);
        return;
    }
    Iterator it = set.iterator();
    while (it.hasNext())
        this.db.deleteDocument(collection, null, ((String) it.next()));
  }
  
}
