/*
Tiedosto: ViewMonitor.java
Luokat:   ViewMonitor, ViewEvent

Tekij:   Olli Pettay
Luonti:   17.11.2002


Muutokset:
  17.11.2002 Olli Pettay  
              Luokan luonti
  04.12.2002 Olli Pettay
              Toteutusta
   

*/


package converge.kernel;
import converge.service.Message;
import org.w3c.dom.Document;
//import converge.service.Message;
/**
*Tmn luokan ilmentymi kytetn nkymien muutosten tekemisess.
*ViewMonitor olioita luodaan vain ViewManager-luokan getNewViewMonitor-metodissa,
*mutta kytt helpottamaan on tehty staattisia metodeja, jolloin ei tarvitse 
*vlitt ViewMonitor-olioiden luonnista.
*
*@see converge.kernel.ViewManager#getNewViewMonitor
*/
public class ViewMonitor {
  
  private ViewEvent event;
  private View view;
  private Document dom;
  private String[] viewNames;
  private boolean notFound;
  
  protected ViewMonitor(){
    this.event = null;
    this.view = null;
    this.notFound = false;
    this.viewNames = null;
  }
  
  /**
  *ViewManagerThread odottaa kunnes olioon listty tapahtuma.
  *<i>Vain ytimen kyttn</i>
  */
  protected synchronized void waitEvent() {
    while(this.event == null)
      try {
        wait();
      }
      catch(Exception e){;}
    
  }
  
  /**
  *Palautetaan tapahtuma.
  *<i>Vain ytimen kyttn</i>
  */
  protected synchronized ViewEvent getEvent() {
    return event;
  }
  
  /**
  *ViewManagerThread asettaa nkymn.
  *<i>Vain ytimen kyttn</i>
  */
  protected synchronized void setView(View view) {
    this.view = view;
    if (view == null)
      this.notFound = true;
    notify();
  }
  
  
  /**
  *ViewManagerThread asettaa dokumentin.
  *<i>Vain ytimen kyttn</i>
  */
  protected synchronized void setDom(Document dom) {
    this.dom = dom;
    if (dom == null)
      this.notFound = true;
    notify();
  }
  
  /**
  *ViewManagerThread asettaa nkymien nimet.
  *<i>Vain ytimen kyttn</i>
  */
  protected synchronized void setViewNames(String[] names) {
    this.viewNames = names;
    if (names == null)
      this.notFound = true;
    notify();
  }
  
  
  //////////////////////////////////////////////////////////////////////////
  
  private static ViewMonitor newViewMonitor() {
    return ViewManager.getInstance().getNewViewMonitor();
  }
  
  
  //Tm on vain ytimelle
  /**
  *Asetetaan viesti johonkin nkymn ja tallennetaan mys itse viesti.
  *
  *@param msg Viestiolio
  *@param view Nkym
  */
  
  public static void setMessageToView(Message msg, String view) {
    ViewMonitor vm = newViewMonitor();
    if (vm != null)
      vm.setMessageToView_(msg, view);
  }
  
  private synchronized void setMessageToView_(Message msg, String view) {
    this.event = new ViewEvent(ViewEvent.SET_MESSAGE ,msg, view);
    this.notify();
  }
  
  
  
  /**
  *Asetetaan viesti nkymn
  *
  *@param user Kyttj
  *@param id Viestin ID
  *@param view Nkymn nimi
  */
  public static void setMessageToView(User user, String id, String view) {
     ViewMonitor vm = newViewMonitor();
    if (vm != null)
      vm.setMessageToView_(user, id, view);
  }
  
  private synchronized void setMessageToView_(User user, String id, String view) {
    this.event = new ViewEvent(user , ViewEvent.SET_MESSAGE_BY_ID , id, view);
    this.notifyAll();
  }
  
  /**
  *Poistetaan viesti nkymst, ja mikli viesti oli liitetty vain
  *yhteen nkymn, poistetaan se mys kannasta.
  *
  *@param user Kyttj
  *@param id Viestin ID
  *@param view Nkymn nimi
  */
  public static void deleteMessageFromView(User user, String id, String view) {
     ViewMonitor vm = newViewMonitor();
    if (vm != null)
      vm.deleteMessageFromView_(user, id, view);
  }
  
  private synchronized void deleteMessageFromView_(User user, String id, String view) {
    this.event = new ViewEvent(user, ViewEvent.DELETE_MESSAGE_FROM_VIEW, id, view);
    this.notifyAll();
  }
  
  /**
  *Poistetaan viesti kannasta ja jokaisesta nkymst
  *
  *@param user Kyttj
  *@param id Viestin ID
  */
  
  public static void deleteMessage(User user, String id) {
    ViewMonitor vm = newViewMonitor();
    if (vm != null)
      vm.deleteMessage_(user, id);
  }
  
  private synchronized void deleteMessage_(User user, String id) {
    this.event = new ViewEvent(user, ViewEvent.DELETE_MESSAGE_TOTAL, id);
    this.notifyAll();
  }
  
  
  /**
  *Palautetaan nkym
  *
  *@param user Kyttj
  *@param view Nkymn nimi
  *@return Nkym tai null, jos ei lytynyt 
  */
  
  public static View getView(User user,String view) {
    ViewMonitor vm = newViewMonitor();
    if (vm != null)
      return vm.getView_(user, view);
    return null;
  }
  
  private synchronized View getView_(User user,String view) {
    this.event = new ViewEvent(user, ViewEvent.GET_VIEW,view);
    notifyAll();
    while (this.view == null && this.notFound == false) {
      try {
        wait();
      }
      catch (Exception e){;}
    }
    return this.view;
  }
  
  /**
  *Viestin haku
  *
  *@param user Kyttj
  *@param id Viestin ID
  */
  public static org.w3c.dom.Document getMessage(User user, String id) {
    ViewMonitor vm = newViewMonitor();
    if (vm != null)
      return vm.getMessage_(user, id);
    return null;
  }
  
   private synchronized org.w3c.dom.Document getMessage_(User user, String id) {
     this.event = new ViewEvent(user, ViewEvent.GET_MESSAGE_BY_ID, id);
     notifyAll();
     while (this.dom == null && this.notFound == false) {
       try {
        wait();
      }
      catch(Exception e){;}
    }
    return this.dom;
  }
  
  /**
  *Haetaan kyttjn nkymien nimet
  *
  *@param user Kyttj
  *@return taulukko nimist tai null, jos ei lytynyt
  */
  
  public static String[] getViewNames(User user) {
    ViewMonitor vm = newViewMonitor();
    if (vm != null)
      return vm.getViewNames_(user);
    return null;
  }
  
  private synchronized String[] getViewNames_(User user) {
    this.event = new ViewEvent(user, ViewEvent.GET_VIEW_NAMES);
    notifyAll();
    while (this.viewNames == null && this.notFound == false) {
      try {
        wait();
      }
      catch (Exception e) {return null;}
    }
    return this.viewNames;
  }
  
  /**
  *Poistetaan nkym
  *
  *@param user Kyttj
  *@param view Nkymn nimi
  */
  public static void deleteView(User user, String view) {
  ViewMonitor vm = newViewMonitor();
    if (vm != null)
      vm.deleteView_(user, view);
  }
  
  private synchronized void deleteView_(User user, String view) {
    this.event = new ViewEvent(user, ViewEvent.DELETE_VIEW, view);
    notifyAll();
  }
  

  
    
}

class ViewEvent {
  protected static final int SET_MESSAGE_BY_ID         = 1;
  protected static final int SET_MESSAGE               = 2;
  protected static final int DELETE_MESSAGE_FROM_VIEW  = 3;
  protected static final int DELETE_MESSAGE_TOTAL      = 4;
  protected static final int GET_VIEW                  = 5;
  protected static final int GET_MESSAGE_BY_ID         = 6;
  protected static final int GET_VIEW_NAMES            = 7;
  protected static final int DELETE_VIEW               = 8;
  
  public Message message = null;
  public String id = null;
  public String view = null;
  public User user = null;
  public int type = 0;
  public ViewEvent(int type, Message msg, String view) {
    this.message = msg;
    this.id = msg.getMessageID();
    this.view = view;
    this.type = type;
    this.user = msg.getUser();
  }
  
  public ViewEvent(User user, int type, String msg, String view) {
    this.id = msg;
    this.view = view;
    this.type = type;
    this.user = user;
  }
  public ViewEvent(int type, Message msg) {
    this.message = msg;
    this.type = type;
    this.user = msg.getUser();
  }
  
  public ViewEvent(User user, int type, String msg) {
    if (type == GET_VIEW || type == DELETE_VIEW)
      this.view = msg;
    else
      this.id = msg;
    this.type = type;
    this.user = user;
  }
  
  public ViewEvent(User user, int type) {
    this.user = user;
    this.type = type;
  }
  
}
