/*
Tiedosto: Attachment.java
Luokat:   Attachment

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
Luokka liitetiedostojen hallintaan.
<p>
Kaikista liitetiedostoista tallennetaan tietokantaan ainoastaan liitetiedoston osoite.
Jos viestin mukana tulee perinteinen liitetiedosto, se korvataan pointterilla
IMAP-postilaatikkoon, jolloin URLiksi tulee esim.
imap://kayttaja@imap.palvelin.fi/<pine.osf.4.30.1322423@sirppi.helsinki.fi>/1-liitetiedostonnimi.txt
<p>
Jos viestin liitetiedostona tulee mrtynmuotoinen osoitteen sisltm tekstitiedosto,
asetetaan viestin liitetiedoston osoitteeksi saapunut osoite.
Tekstitiedosto noudattaa mm. Internet Explorerin kyttm url-tiedostomuotoa.
<br>
<pre>
[InternetShortcut]
URL=http://miuku.net/
Modified=10DA2746B38FC2015C
</pre>

<a href="http://msdn.microsoft.com/library/default.asp?url=/library/en-us/shellcc/platform/shell/programmersguide/shell_int/shell_int_programming/shortcuts/internet_shortcuts.asp">listietoja</a>
*/

public class Attachment
implements converge.kernel.XMLContentInterface {

    private AttachmentAttributes attributes;
    private String URL;

    /**
    * Luo uuden liitetiedosto-olion, jonka tietosislt lytyy osoitteesta URL
    *
    * @param URL   Liitetiedoston pointteri (http tai IMAP tai ...)
    */
    public Attachment(String URL) {
        this.attributes = new AttachmentAttributes();
        this.URL=URL;
        //refreshAttachmentAttributes? sikeeseen?
    }

    /**
    * Noutaa liitetiedoston. Liitetiedoston osoite on jo Attachment-olion tiedossa
    * (URL). Palauttaa PipeMessagen joka pit sislln mahdollisen virheilmoituksen
    * ja pipen josta liitetiedoston voi lukea. Kynnist sikeen liitetiedoston hakemiseksi.
    *
    * @param user   viestin omistajan User-olio
    * @return       putki ja virheilmoitus -olio PipeMessage
    */
    public PipeMessage getAttachment(converge.kernel.User user) {
        try {
            if(this.URL.substring(0,7).equals("imap://")) {
                //IMAP
                // Tm on sinns turha (pelkk ds.canHandle() hoitaisi tarkistukset)
                // eli tnne ei tarvitsisi listata kaikkia tuettuja protokollia,
                // mutta nin saadaan kyttjlle mahdollisesti parempi virheilmoitus
                // virheellisest URLista..
                if(user == null)
                    return new PipeMessage("No user object given to attachment retriever");
                IMAPURLProperties iup = new IMAPURLProperties(this.URL);
                if(iup.protocol == null)
                    return new PipeMessage("Invalid attachment pointer, URL="+
                    webcore.HtmlUtils.convertTag2Entity(this.URL));
                String[] dsNames = user.getDataSourceNames();
                for(int i=0;i<dsNames.length;i++) {
                    DataSource ds=user.getDataSource(dsNames[i]);
                    if(ds.canHandle(this.URL))
                        return ds.getAttachment(this.URL,false);
                        //Kutsutaan HTTP:n tai IMAPin getAttachment-metodia,
                        //joka palauttaa PipeMessagen (sis. putki ja mahd. virheilmo)
                        //ja kynnist sikeen liitetiedoston hakemiseksi.
                        //Palautetusta PipeMessagesta clientin pitisi katsoa
                        //onko tullut virhe, ja jos ei, luetaan PipeMessagen
                        //PipedInputStreamista tavaraa.
                }
                return new PipeMessage("No suitable IMAP server found, URL="+
                    webcore.HtmlUtils.convertTag2Entity(this.URL));

            } else if(this.URL.substring(0,7).equals("http://") || this.URL.substring(0,8).equals("https://")) {
                return new PipeMessage("You should download HTTP and HTTPS attachments directly");
            } else
                return new PipeMessage("Unknown protocol, URL="+
                    webcore.HtmlUtils.convertTag2Entity(this.URL));
        } catch(Exception e) {
            return new PipeMessage("Attachment error: " + e);
        }
    }


    /**
    * Kertoo liitetiedoston ominaisuudet (koko, pivmr, tyyppi ym).
    *
    * @return AttachmentAttributes-olio
    */
    public AttachmentAttributes getAttachmentAttributes() {
        return attributes;
    }

    /**
    * Asettaa liitetiedoston ominaisuudet (koko, pivmr, tyyppi ym).
    *
    * @param aa AttachmentAttributes-olio
    */
    public void setAttachmentAttributes(AttachmentAttributes aa) {
        this.attributes=aa;
    }

    /**
    * Pivitt Attachment-olion ominaisuudet vastaamaan tmnhetkist tilannetta.
    *
    * @param user   viestin omistajan User-olio, tulevaisuutta varten (ei tarvita nykyisill DataSourceilla)
    * @return true jos tiedot muuttuivat, false muulloin
    */
    public boolean refreshAttachmentAttributes(converge.kernel.User user) {
        if(this.URL.length() > 8 && (
               this.URL.substring(0,7).equals("http://") || this.URL.substring(0,8).equals("https://"))) {
            AttachmentAttributes newa = HTTP.getAttachmentAttributes(this.URL);
            boolean unmodified = this.attributes.isSame(newa);
            if(unmodified)
                return false;
            else {
                this.attributes = newa;
                return true;
            }
        }
        return false; //virheellinen osoite
    }

    /**
    * Kertoo Attachment-olion sisltmn liitetiedoston osoitteen.
    *
    * @return liitetiedoston osoite
    */
    public String getPointerAddress() {
        return this.URL;
    }

    /**
    * Muodostaa Attachment-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        Document doc = parent.getOwnerDocument();
        Element attach = doc.createElement("attachment");
        parent.appendChild(attach);
        Element attachURL = doc.createElement("url");
        attachURL.setAttribute("address",this.URL);
        attach.appendChild(attachURL);
        this.attributes.createXML(attach);
    }

    /**
    * Muodostaa annetusta XML-elementist Attachment-olion.
    *
    * @param myContent XML-dokumentin node, jonka lapsielementeist muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        Element content = (Element)myContent;
        if(!content.getTagName().equals("attachment"))
            return false;

        Element child = (Element) content.getFirstChild();
        while (child != null) {
            if(child.getTagName().equals("url")) {
                this.URL=child.getAttribute("address");
            } else if(child.getTagName().equals("attributes")) {
                AttachmentAttributes aa = new AttachmentAttributes();
                if(aa.loadXML(child))
                    this.attributes=aa;
            }
            child = (Element) child.getNextSibling();
        }
        return true;
    }
}
