/*
Tiedosto: ConvergeSSLSocketFactory.java
Luokat:   ConvergeSSLSocketFactory

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;
import javax.net.ssl.TrustManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import javax.net.SocketFactory;

/**
* ConvergeSSLSocketFactory luo SSL-suojattuja verkkosocketeja. Nit tarvitaan haettaessa
* viestej SSL-suojatulta IMAP-palvelimelta tai SSL-suojatulta WWW-palvelimelta.
*/

public class ConvergeSSLSocketFactory extends SSLSocketFactory {

    public SSLSocketFactory factory;

    /**
    * Oletuskonstruktori. Asettaa SSLContextin kyttmn ConvergeTrustManageria
    * sertifikaattien hallitsemiseen.
    */
    public ConvergeSSLSocketFactory() {
        try {
            SSLContext sslcontext = SSLContext.getInstance("TLS");
            sslcontext.init(null, new TrustManager[] {
                (TrustManager) new ConvergeTrustManager() },
                new java.security.SecureRandom());
            factory = (SSLSocketFactory) sslcontext.getSocketFactory();
        } catch(Exception e) {
            System.out.println("CSSLSF exception: " + e);
        }
    }

    /**
    * Palauttaa uuden SSL Socket Factoryn.
    *
    * @return uusi socket factory
    */
    public static SocketFactory getDefault() {
        return new ConvergeSSLSocketFactory();
    }

    /**
    * Luo uuden socketin.
    *
    * @param socket olemassaoleva socket
    * @param server palvelimen osoite
    * @param port palvelimen portti
    * @param autoClose suljetaanko allaoleva socket kun tm socket suljetaan
    * @return luotu verkkosocket
    */
    public Socket createSocket(Socket socket, String server, int port, boolean autoClose)
        throws IOException {
        return factory.createSocket(socket, server, port, autoClose);
    }

    /**
    * Luo uuden socketin.
    *
    * @param inaddr1 palvelimen osoite
    * @param port1 palvelimen portti
    * @param inaddr2 oman koneen osoite
    * @param port2 oman koneen portti
    * @return luotu verkkosocket
    */
    public Socket createSocket(InetAddress inaddr1, int port1, InetAddress inaddr2, int port2)
        throws IOException {
        return factory.createSocket(inaddr1, port1, inaddr2, port2);
    }

    /**
    * Luo uuden socketin.
    *
    * @param inaddr palvelimen osoite
    * @param port palvelimen portti
    * @return luotu verkkosocket
    */
    public Socket createSocket(InetAddress inaddr, int port)
        throws IOException {
        return factory.createSocket(inaddr, port);
    }

    /**
    * Luo uuden socketin.
    *
    * @param server palvelimen osoite
    * @param port1 palvelimen portti
    * @param inaddr oman koneen osoite
    * @param port2 oman koneen portti
    * @return luotu verkkosocket
    */
    public Socket createSocket(String server, int port1, InetAddress inaddr, int port2)
        throws IOException {
        return factory.createSocket(server, port1, inaddr, port2);
    }

    /**
    * Luo uuden socketin.
    *
    * @param server palvelimen osoite
    * @param port palvelimen portti
    * @return luotu verkkosocket
    */
    public Socket createSocket(String server, int port)
        throws IOException {
        return factory.createSocket(server,port);
    }

    /**
    * Palauttaa listan oletuksena kytetyist salakirjoitusmenetelmist.
    *
    * @return salakirjoitusmenetelmien taulukko
    */
    public String[] getDefaultCipherSuites() {
        return factory.getSupportedCipherSuites();
    }

    /**
    * Palauttaa listan tuetuista salakirjoitusmenetelmist.
    *
    * @return salakirjoitusmenetelmien taulukko
    */
    public String[] getSupportedCipherSuites() {
        return factory.getSupportedCipherSuites();
    }
}
