/*
Tiedosto: Email.java
Luokat:   Email

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;
import javax.mail.Part;
import javax.mail.Header;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Folder;
import javax.mail.Store;
import java.util.Enumeration;
import java.util.Date;
import java.io.PipedOutputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import com.sun.mail.imap.IMAPNestedMessage;
import converge.kernel.Log;

/**
* Email on luokka, joka osaa muuntaa saamansa Internetin shkpostiviestin
* jrjestelmn sisiseen muotoon.
*/
public class Email {

    /**
    * Muuntaa shkpostiviestin jrjestelmn sisiseen muotoon.
    *
    * @param user  Message-olioon listtv User-olio
    * @param inmsg saapunut shkpostiviesti
    * @param URLprefix liitetiedoston osoitteen alkuosa, sis. protokollan, kyttjn ja palvelimen nimen
    * @return viesti jrjestelmn sisisess muodossa, null jos muodostaminen ei onnistu
    */
    public static converge.service.Message convertMessage(converge.kernel.User user,
            javax.mail.internet.MimeMessage inmsg, String URLprefix) {
        if(inmsg == null)
            return null;
        Message outmsg = new Message(user);
        outmsg.setMessageAttribute("Date",new Date(),true); // oletusarvo
        try {
            for (Enumeration e = inmsg.getAllHeaders(); e.hasMoreElements();) {
                Header h = (Header)e.nextElement();
                String name = h.getName(), value = h.getValue();
                if(name.equals("Message-Id"))
                    name="Message-ID";
                if(name.equals("Message-ID"))
                    URLprefix += value;
                if(name.equals("Cc") || name.equals("cc"))
                    name="CC";
                if(name.equals("From") || name.equals("To") || name.equals("Subject")
                   || name.equals("CC"))
                    value = javax.mail.internet.MimeUtility.decodeText(value);
                if(name.equals("From") || name.equals("To") || name.equals("Subject")
                   || name.equals("Message-ID") || name.equals("CC") || name.equals("Date")
                   || name.equals("X-Priority") || name.equals("Precedence"))
                    if(name.equals("Date"))
                        try {
                            outmsg.setMessageAttribute(name,inmsg.getSentDate(),true);
                        } catch(Exception datee) {}
                    else
                        outmsg.setMessageAttribute(name,value,true);
                else
                    outmsg.setMessageAttribute(name,value,false);
            }
            /* viestin body */
            Object content = inmsg.getContent();
            if (content instanceof Multipart) {
                handleMultipart((Multipart)content,outmsg,URLprefix);
            } else {
                handlePart(inmsg,outmsg,0,URLprefix);
            }
        } catch(Exception e) {
            System.out.println("Email exception " + e);
            Log.stack(e,"Email");
        }
        return outmsg;
    }

    /**
    * Ksittelee moniosaiset viestit. Kutsuu handlePart-metodia jokaiselle viestin osalle.
    *
    * @param multipart moniosainen shkpostiviesti
    * @param outmsg    uusi viesti jrjestelmn sisisess muodossa
    * @param URLprefix liitetiedoston osoitteen alkuosa, sis. protokollan, kyttjn ja palvelimen nimen
    */
    public static void handleMultipart(Multipart multipart, Message outmsg, String URLprefix)
        throws MessagingException, IOException {
        for (int i=0, n=multipart.getCount(); i<n; i++) {
            handlePart(multipart.getBodyPart(i),outmsg,i,URLprefix);
        }
    }

    /**
    * Ksittelee yhden viestin osan, tai koko viestin jos osia on vain yksi.
    *
    * @param part   viestin osa
    * @param outmsg uusi viesti jrjestelmn sisisess muodossa
    * @param URLprefix liitetiedoston osoitteen alkuosa, sis. protokollan, kyttjn ja palvelimen nimen
    */
    public static void handlePart(Part part, Message outmsg, int partNum, String URLprefix)
      throws MessagingException, IOException {
        String disposition=null,contentType=null;
        boolean attributesHaveBeenSet=false;
        try {
            disposition = part.getDisposition();
        } catch(MessagingException me) {}
        try {
            contentType = part.getContentType();
        } catch(MessagingException me) {
            contentType = "text/plain";
        }
        if (disposition == null) {   // vain body
            if ((contentType.length() >= 10) &&
                (contentType.toLowerCase().substring(
                 0, 10).equals("text/plain"))) {
                // Viestin sislt pelkkn tekstin
                Object content = part.getContent();
                if(content instanceof IMAPNestedMessage)
                    // yleens virheilmoituksia, voisi ksitell laajemminkin ...
                    outmsg.setMessageAttribute("MESSAGE_BODY","Unable to deliver the following message:\n\n" + ((IMAPNestedMessage)content).getContent(),true);
                else
                    outmsg.setMessageAttribute("MESSAGE_BODY",content,true);

            } else {
                Object content = part.getContent();
                if(content instanceof IMAPNestedMessage)
                    // yleens virheilmoituksia, voisi ksitell laajemminkin ...
                    outmsg.setMessageAttribute("MESSAGE_BODY","Unable to deliver the following message:\n\n" + ((IMAPNestedMessage)content).getContent(),true);
                else
                    outmsg.setMessageAttribute("MESSAGE_BODY",content,true);            }
        } else if (disposition.equalsIgnoreCase(Part.ATTACHMENT) || disposition.equalsIgnoreCase(Part.INLINE)) {
            AttachmentAttributes aa = new AttachmentAttributes();
            try {
                aa.setSize(part.getSize());
            } catch(MessagingException me) {
                aa.setSize(-1);
            }
            aa.setDate(new Date());
            aa.setMIMEType(contentType);
            try {
                aa.setDescription(part.getDescription());
            } catch(MessagingException me) {
                aa.setDescription("");
            }
            String filename="unknown.txt";
            try {
                filename = part.getFileName();
// name=mm.txt; charset=ISO-8859-1
                if(filename.substring(0,4).equals("name=") && filename.indexOf(";")>0) {
//                    System.out.println("EMAIL: old 'file'name = " + filename);
                    filename=filename.substring(5,filename.indexOf(";")-1);
//                    System.out.println("EMAIL: new 'file'name = " + filename);
                }
            } catch(MessagingException me) {}
//            System.out.println("EMAIL: new filename = " + filename);
            aa.setFilename(filename != null ? filename : "unknown.txt");
            Attachment a = null;
            if(filename.lastIndexOf(".url") == filename.length()-4) {
                System.out.println("EMAIL: Tiedostonimi pttyy .url:iin, ksitelln pointterina");
/*
[DEFAULT]
BASEURL=http://news.google.com/
[InternetShortcut]
URL=http://news.google.com/
Modified=F0EFFF2169A3C201CE
*/
                BufferedReader reader = null;
                try {
                    reader = new BufferedReader(new InputStreamReader(part.getInputStream()));
                    boolean foundInternetShortcut = false;
                    String line;
                    for(int row=0;(line = reader.readLine())!=null &&
                        row < 10 && a == null;row++) {
                        if(line.length() >= 18 && line.substring(0,18).equals("[InternetShortcut]"))
                            foundInternetShortcut = true;
                        if(foundInternetShortcut && line.length()>= 4 && line.substring(0,4).equals("URL=")) {
                            a = new Attachment(line.substring(4,line.length()));
                            System.out.println("EMAIL: new pointer: " + line.substring(4,line.length()));
                        }
                    }
                    try {
                        reader.close();
                    } catch(Exception ee) {}

                    if(a!=null) {
                        a.refreshAttachmentAttributes(null);
                        attributesHaveBeenSet=true;
                        a.getAttachmentAttributes().setFilename(filename.substring(0,filename.length()-4));
                    } else
                        a = new Attachment(URLprefix + "/" + partNum + "-" + filename);

                } catch(Exception e) {
                    Log.stack(e,"Email");
                    a = new Attachment(URLprefix + "/" + partNum + "-" + filename);
                    try {
                        if(reader != null)
                            reader.close();
                    } catch(Exception ee) {}
                }
            } else {
                // URLprefix = imap://tunnus@palvelin/<messageviesti432874298742>
                // URL       = imap://tunnus@palvelin/<messageviesti432874298742>/1-liitetiedosto.txt
                a = new Attachment(URLprefix + "/" + partNum + "-" + filename);
            }
            if(!attributesHaveBeenSet)
                a.setAttachmentAttributes(aa);
            outmsg.addAttachment(a);

            System.out.println("EMAIL: Handlepart: " + part.getFileName() +
                               " : " + contentType + ", size: " + part.getSize() + ", descr: " +
                                  part.getDescription());
        } else {  // ei pitisi tulla nit koskaan
           System.out.println("EMAIL: Other email part: " + disposition);
        }
    }

    /**
    * Noutaa liitetiedoston.
    *
    * @param p moniosaisen viestin yksi osa
    * @param shouldBeFileName liitetiedoston oletettu nimi
    * @param folder IMAP-kansio, vlitetn vain sulkemista varten
    * @param store IMAP-viestisil, vlitetn vain sulkemista varten
    * @param testOnly tarkistetaanko vain liitetiedoston olemassaolo
    * @return viestiolio joka kertoo liitetiedostoa noudettaessa tulleen virheilmoituksen, tai
    *         tiedostokahvan josta kutsuva metodi voi liitetiedoston lukea.
    */
    // folder ja store vain sulkemista varten
    public static PipeMessage getAttachment(Part p, String shouldBeFileName, Folder folder, Store store,
                            boolean testOnly) {
        final int testsize = 4*1024;
        final int bufsize = 16*1024;
        int readbytes = 0;
        char[] buffer = new char[testsize];
        String disposition=null;
        PipeMessage pm = null;

        try {
            disposition = p.getDisposition();
        } catch(MessagingException me) {
            try {
                folder.close(false);
                store.close();
            } catch(Exception e) {};
            return new PipeMessage("Unable to figure out attachment disposition");
        }
        if (disposition == null) {   // vain body
            try {
                folder.close(false);
                store.close();
            } catch(Exception e) {};
            return new PipeMessage("Attachment pointer pointed to message body");
        } else if (disposition.equalsIgnoreCase(Part.ATTACHMENT) || disposition.equalsIgnoreCase(Part.INLINE)) {
            String filename="unknown.txt";
            try {
                filename = p.getFileName();
// name=mm.txt; charset=ISO-8859-1
                if(filename.substring(0,4).equals("name=") && filename.indexOf(";")>0) {
                    System.out.println("EMAIL: old 'file'name = " + filename);
                    filename=filename.substring(5,filename.indexOf(";")-1);
                    System.out.println("EMAIL: new 'file'name = " + filename);
                }
            } catch(MessagingException me) {}
            System.out.println("EMAIL: new filename = " + filename);
            filename = filename != null ? filename : "unknown.txt";
            if(!(filename.equals(shouldBeFileName))) {
                try {
                    folder.close(false);
                 store.close();
                } catch(Exception e) {};
                return new PipeMessage("Attachment filename incorrect ("
                    + filename + "<->" + shouldBeFileName + ")");
            }
        }
        if(testOnly)
            return new PipeMessage("");

        InputStream is;
        InputStreamReader isr;
        BufferedReader bufr;
        try {
            is = p.getInputStream();
            isr = new InputStreamReader(is);
            bufr = new BufferedReader(isr,bufsize);
        } catch(Exception e) {
            try {
                folder.close(false);
                store.close();
            } catch(Exception e2) {};
                return new PipeMessage("Unable to open attachment: "+e);
        }

        PipedOutputStream outpipe = null;
        OutputStreamWriter osw = null;
        BufferedWriter bufw = null;

        try {
            outpipe = new PipedOutputStream();
            osw = new OutputStreamWriter(outpipe);
            bufw = new BufferedWriter(osw,bufsize);
            pm = new PipeMessage(outpipe,"");
        } catch(Exception e) {
                try {
                    bufw.close();
                    osw.close();
                    outpipe.close();
                    folder.close(false);
                    store.close();
                } catch(Exception e2) {};
                return new PipeMessage("Unable to open internal transfer pipes: "+e);
        }

        try {
            readbytes = bufr.read(buffer,0,testsize);
        } catch(Exception e) {
            try {
                folder.close(false);
                store.close();
            } catch(Exception e2) {};
            return new PipeMessage("Error while reading the attachment from IMAP server");
        }

        try {
            bufw.write(buffer,0,readbytes);
        } catch(Exception e) {
            try {
                bufw.close();
                osw.close();
                outpipe.close();
                folder.close(false);
                store.close();
            } catch(Exception e2) {};
            return new PipeMessage("Error writing to the output pipe");
        }

        // kynnistetn sie varsinaista tiedoston siirtoa varten
        IMAPStreamDownloader sd = new IMAPStreamDownloader(bufr,bufw,isr,osw,is,outpipe,folder,store);
        sd.start();
        return pm; // kaikki ok, sie jatkaa nyt siirtoa ja client-puoli voi alkaa puskea tiedostoa WWW-selaimelle
    }
}
