/*
Tiedosto: IMAPMessageThread.java
Luokat:   IMAPMessageThread

Tekij:   Anssi Johansson
Luonti:   9.12.2002

Muutokset:
  09.12.2002  Luokan luonti

*/
package converge.service;

import java.util.Date;
import java.util.Properties;
import javax.mail.Folder;
import javax.mail.Flags;
import javax.mail.Store;
import javax.mail.Session;
import java.security.Security;
import converge.kernel.Log;

/**
* Sie viestien hakemiseksi IMAP-postilaatikosta
*/

public class IMAPMessageThread
extends Thread {

    private IMAP imap;
    private converge.kernel.User user;

    /**
    * Oletuskonstruktori.
    *
    * @param imap   IMAP-olio
    * @param user   kyttjolio
    */
    public IMAPMessageThread(IMAP imap, converge.kernel.User user) {
        this.imap=imap;
        this.user=user;
    }

    /**
    * Kynnist sikeen viestien hakemiseksi
    */
    public void run() {
        try {
            Session session;
            if(this.imap.useSSL) {
                final String SSL_FACTORY = "converge.service.ConvergeSSLSocketFactory";
                Security.setProperty("ssl.SocketFactory.provider","ConvergeSSLSocketFactory");
                Properties props = System.getProperties();
                props.setProperty("mail.imap.socketFactory.class",SSL_FACTORY);
                /* jos palvelin ei ymmrr SSL: niin ei yritet kytt suojaamatonta protokollaa */
                props.setProperty("mail.imap.socketFactory.fallback","false");
                props.setProperty("mail.imap.port","993");
                props.setProperty("mail.imap.socketFactory.port","993");
                session = Session.getInstance(props);
            } else
                session = Session.getInstance(new Properties(), null);
            Store store = session.getStore("imap");
            store.connect(this.imap.serverAddress,this.imap.serverLogin,this.imap.serverPassword);

            Folder folder = store.getFolder("INBOX"); // tm voi olla joskus jotain muutakin!

            // okei, pstiin sislle, nollataan virhelaskuri
            this.imap.consecutiveFailures=0;
            this.imap.setErrorMessage(null);

            if(!folder.hasNewMessages()) {
                store.close();
                return;
            }
            folder.open(Folder.READ_WRITE);

            javax.mail.Message message[] = folder.getMessages();
            for(int i=0, n=message.length;i<n;i++) {
                if(message[i].isSet(Flags.Flag.SEEN))
                    continue;
                String URLprefix = "imap://"+this.imap.serverLogin+"@"+this.imap.serverAddress+"/";

                Message msg = Email.convertMessage(this.user, (javax.mail.internet.MimeMessage)message[i],URLprefix);
                if(msg != null) {
                    // DEBUG
                    System.out.println("IMAPMT: Received a new message.");
                    converge.kernel.ContextModelHandler.incomingMessage(msg);
                    message[i].setFlag(Flags.Flag.SEEN,true);
                }

//DEBUG ************************************
/*        System.out.println("IMAPMT: alkuperinen viesti XML:n");
        org.w3c.dom.Document doc = converge.kernel.XMLUtil.newXMLDocument();
        msg.createXML(doc);
        converge.kernel.XMLUtil.outputPrinting(doc,System.out); */

/*        System.out.println("IMAPMT: Viestin kopio XML:n");
        Message msg2 = new Message(null,doc);
        org.w3c.dom.Document doc2 = converge.kernel.XMLUtil.newXMLDocument();
        msg2.createXML(doc2);
        converge.kernel.XMLUtil.outputPrinting(doc2,System.out); */

//DEBUG **********************************************

            }
            folder.close(false);
            store.close();
        } catch(Exception e) {
            this.imap.consecutiveFailures++; // kasvatetaan virhelaskuria
            this.imap.setErrorMessage(e.getMessage());
//            System.out.println("IMAPMT exception: " + e);
            Log.stack(e,"IMAPMessageThread");
            Log.stack(e,"out");
            Message m = new Message(this.user);
            m.setMessageAttribute("From","The Friendly Converge System Administrators",true);
            m.setMessageAttribute("To","You",true);
            m.setMessageAttribute("Subject","[CONVERGE] IMAP server error",true);
            m.setMessageAttribute("Date",new Date(),true);
            m.setMessageAttribute("MESSAGE_BODY",
                "Hi. This is the Converge System at University of Helsinki, Department\n" +
                "of Computer Science. I'm afraid I wasn't able to pick up your new\n" +
                "email from " + this.imap.serverAddress + ", also known as " + this.imap.dataSourceName +".\n" +
                "The reason for the mail pickup failure was:\n\n"+
                e.getMessage() +
                "\n\nIf this looks like an error that you can fix, feel free to fix your\n" +
                "settings accordingly. On the other hand, if it looks like an internal\n" +
                "system error, you're in big trouble. Sorry it didn't work out.\n\n" +
                "-- \nThe Friendly Converge System Administrators",true);
            converge.kernel.ContextModelHandler.incomingMessage(m);
        }
    }
}
