/*
Tiedosto: Message.java
Luokat:   Message

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002 Anssi Johansson
             Luokan luonti
  06.12.2002 Anssi Johansson
             Perustoiminnallisuutta
  05.12.2002 Mikko Hiipakka
             Viestiolion id muutettu kentksi "private String messageID".
             setAttribute toteutusta muutettu kyttmn AttributeLoaderia.
             addCMValueAttribute-metodi listty.
  10.12.2002 Anssi Johansson
             XML-jutut
  15.12.2002 Mikko Hiipakka
             Rakennetta tarkistettu
*/

package converge.service;
import converge.kernel.Log;
import converge.kernel.User;
import converge.attribute.*;
import converge.attribute.attributeImplementations.*;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.util.Date;

/**
* Message sislt viestin jrjestelmn sisisess muodossa.
*/

public class Message
implements converge.kernel.XMLContentInterface {

    //Jokaiselle viestille yksiksitteinen id
    private String messageID = "";
    //Viestin vastaanottajaolion ilmentym
    private converge.kernel.User user;
    //Attribuutti-ilmentymien silytys
    private java.util.HashMap attr = new java.util.HashMap();
    //Loogiset attribuuttien nimet, jotka ovat tarkoitettu kantaan talletettavaksi
    private java.util.HashSet attrPerm = new java.util.HashSet();
    private Attachment[] attachments = new Attachment[0];

    /**
    * Luo uuden Message-olion annetusta XML-dokumentista.
    * @param user kyttjolio
    * @param doc  XML-dokumentti
    */
    public Message(converge.kernel.User user, org.w3c.dom.Document doc) {
        this.user=user;
        if ( this.loadXML(doc) )
            Log.output("Message").println("Ilmentym ladattu XML:st. ( ID : " + this.messageID + " )");
        else
            Log.output("Message").println("Ilmentymn lataus XML:st eponnistui "
                                          + "( ID - " + this.messageID + " )"  );
    }

    /**
    * Luo uuden tyhjn Message-olion.
    *
    * @param user kyttjolio
    */
    public Message(converge.kernel.User user) {
        messageID = Integer.toHexString(("koe" + new Date()).hashCode()) + "-"
                  + Integer.toHexString((int)(Math.random()*100000000));
        this.user = user;
        Log.output("Message").println("Ilmentym luotu. ( ID : " + this.messageID
                                    + " - User : " + (user == null ?  "--null--" : user.getName() + " )" ));
    }

    /**
    * Palauttaa Message-olioon liitetyn User-olion.
    *
    * @return User-olio
    */
    public converge.kernel.User getUser() {
        return this.user;
    }

    /**
    * Palauttaa Messagen yksiksitteisen viestin tunnuksen
    *
    * @return viestin tunnus
    */
    public String getMessageID() {
        return messageID;
    }

    /**
    * Palauttaa halutun viestiattribuutin.
    *
    * @param name viestiattribuutin nimi
    * @return viestiattribuutti tai null
    */
    public MessageAttributeInterface getMessageAttribute(String name) {
        MessageAttributeInterface a = (MessageAttributeInterface)(this.attr.get(name));
        if ( a == null ){
            a = AttributeLoader.newMessageAttributeInstance( name );
            if ( a == null ){
                Log.output("Message-Attributes").println("Attribuuttin Liittminen Eponnistui - "
                                   + "AttributeLoader palautti null " +"( ID : " + this.messageID
                                   +" :  attribuutti : " + name + " )");
                return null;
            }
            a.setMessage( this );
        }
        return a;
    }


    /**
    * Liitetn viestiolioon uusi MessageAttributeInterface:n toteuttava attribuutti,
    *
    * @param name Attribuutti-ilmentymlle annettava nimi, joka on jokin loogisen
    *             attribuutin nimi esim. Subject
    * @param value Attribuutti-ilmentymlle annettava arvotietosislt
    * @param permanent Onko attribuutti-ilmentym viestiin liitettv fakta eli
    *                  tallennetaanko tieto kantaan
    */
    public void setMessageAttribute(String name, Object value, boolean permanent) {
        // Pyydetn nimen perusteella toteutuksen tarjoavan luokkan ilmentym AttributeLoaderilta
//        System.out.println("MESSAGE: Setting attribute " + name);
        MessageAttributeInterface newAttr = AttributeLoader.newMessageAttributeInstance( name );
        if ( newAttr == null ) {
            Log.output("Message-Attributes").println("Attribuuttin Liittminen Eponnistui - "
                          + "AttributeLoader palautti null " + "( ID : " + this.messageID
                          + " :  attribuutti : " + name + " )");
            return;
        }
//        System.out.println("MESSAGE: Setting attribute " + name + " interface found");
        newAttr.setMessage( this );
        newAttr.setValue( value );
        newAttr.setName( name );

        if(permanent)
            attrPerm.add(name);
        else
            attrPerm.remove(name);
        attr.put(name,newAttr);
//        System.out.println(attrPerm.size());
    }


    /**
    * ContextModelValueAttribuuttien viestiolioon lismiseen tarkoitettu metodi,
    *
    * @param cmAttr Parametri, joka listn viestiolion ei kantaan talletettavana attribuuttina
    */
    public void addAttribute( ContextModelValueAttribute cmAttr ){
        attr.put( cmAttr.getName(), cmAttr );
    }

    /**
    * Palauttaa listan viestiin liitetyist liitetiedostoista
    *
    * @return Attachment-taulukko liitetiedostoista
    */
    public Attachment[] getAttachmentList() {
        return attachments;
    }

    /**
    * Lis viestiin uuden liitetiedoston.
    *
    * @param attachment uusi liitetiedosto (Attachment-olio)
    */
    public void addAttachment(Attachment attachment) {
        Attachment[] newatt = new Attachment[this.attachments.length+1];
        int i;
        //Kopioidaan vanhat
        for(i=0;i<this.attachments.length;i++) {
            newatt[i]=this.attachments[i];
        }
        newatt[i]=attachment;
        //Pivitetn muutos
        attachments = newatt;
    }

    /**
    * Muodostaa Message-oliosta XML-elementin.
    *
    * @param parent XML-dokumentin node, jonka lapsielementiksi olion tiedot kirjoitetaan
    */
    public void createXML(org.w3c.dom.Node parent) {
        Document doc = (Document)parent;
        Element root = doc.createElement("message");
        root.setAttribute("id", this.messageID);
        doc.appendChild(root);

        Object[] keys = this.attrPerm.toArray();
        for( int i=0; i<keys.length; i++){
          Element item = doc.createElement("item");
          String key = (String)keys[i];
          item.setAttribute("name",key);
          root.appendChild( item );
          AttributeInterface currentAttr = (AttributeInterface)this.attr.get(key);
          currentAttr.createXML(item);
        }
        for(int i=0;i<attachments.length;i++)
            attachments[i].createXML(root);
    }

    /**
    * Muodostaa annetusta XML-elementist Message-olion.
    *
    * @param myContent XML-dokumentti, jonka sisllst muodostetaan olion tiedot
    * @return true jos tietojen lataus onnistui
    */
    public boolean loadXML(org.w3c.dom.Node myContent) {
        Element content = (Element)((Document)myContent).getFirstChild();
        if(!content.getTagName().equals("message")) {
            Log.output("Message").println("XML lataus eponnistui - Annettun "
                                + "Noden alta ei lytynyt message-elementti");
            return false;
        }
        this.messageID=content.getAttribute("id");

        Element item = (Element) content.getFirstChild();
        while (item != null ) {
          if ( item.getTagName().equals("item") ){
              String name = item.getAttribute("name");
              // Pyydetn nimen perusteella toteutuksen tarjoavan luokkan ilmentym AttributeLoaderilta
              MessageAttributeInterface newAttr = AttributeLoader.newMessageAttributeInstance( name );
              if ( newAttr != null ) {
                  newAttr.loadXML(item.getFirstChild());
                  attrPerm.add( name );
                  attr.put( name ,newAttr);
              }else{
                Log.output("Message-Attributes").println("XML lataus eponnistui - "
                          + "AttributeLoader palautti null ( ID : " + this.messageID
                          + " : attribute : " + name + ")");
                return false;
              }
          }
          if ( item.getTagName().equals("attachment") ){
             Attachment a = new Attachment("");
             a.loadXML(item);
             this.addAttachment(a);
           }
           item = (Element)item.getNextSibling();
        }
        return true;
    }

/*
        <item name="Subject">    HUOM item elementti tytyy luoda jo Messagessa, josta attribuutti ei tied mitn
          <AttributeString>
             <name>Subject</name>
             <value>testiotsikko</value>
          </AttributeString>
        </item>
*/

    //DEBUG *****************************************************************
    public static void main(String[] args) {
      /*
        AttributeLoader.loadClasses();
        converge.kernel.User s = null;

        converge.service.Message msg = new Message(s);
        msg.setMessageAttribute("Subject", new String("Test"), true);
        msg.setMessageAttribute("Message-ID","Test", true);
        msg.setMessageAttribute("CMTESTI", "Test", true);
        System.out.println(msg.getMessageID());
        org.w3c.dom.Document doc = converge.kernel.XMLUtil.newXMLDocument();
        msg.createXML(doc);
        converge.kernel.XMLUtil.outputPrinting(doc,System.out);
        msg = new Message( s,doc );
        Document doc2 = converge.kernel.XMLUtil.newXMLDocument();
        msg.createXML(doc2);
        converge.kernel.XMLUtil.outputPrinting(doc2,Log.output("Message-Test")); */
    }
    //DEBUG *****************************************************************
}
