/*
Tiedosto: SMTP.java
Luokat:   SMTP

Tekij:   Anssi Johansson
Luonti:   3.12.2002

Muutokset:
  03.12.2002  Luokan luonti

*/

package converge.service;
import javax.mail.Message;
import javax.mail.internet.InternetAddress;
import javax.mail.Transport;
import javax.mail.Session;
import javax.mail.internet.AddressException;
import java.util.Date;
import java.util.HashSet;
import java.util.StringTokenizer;
import java.util.Iterator;
import converge.kernel.Log;

/**
* SMTP hoitaa shkpostiviestien lhettmisen.
*/

public class SMTP {

    /**
    * Lhett shkpostiviestin.
    *
    * @param from    viestin lhettjn osoite
    * @param to      viestin vastaanottajan osoite tai osoitteet pilkuilla erotettuna
    * @param subject viestin otsikko
    * @param message viestin teksti
    * @return onnistuiko lhetys
    */
    public static boolean sendMail(String from, String to, String subject, String message, converge.kernel.User user) {
        if(from == null || to == null || subject == null || message == null)
            return false;
        to = to.replaceAll(","," ");
        StringTokenizer st = new StringTokenizer(to);
        HashSet recipHash = new HashSet();
        try {
            while(st.hasMoreTokens()) {
                String one = st.nextToken();
                try {
                    recipHash.add(new InternetAddress(one));
                } catch(AddressException ae) {
                    sendErrorMessage(user,
                        "regarding '" + subject + "' to the following address:\n\n" +
                        one +
                        "\n\nPlease correct the address and try again.\n\n");
                } catch(Exception e) {
                    Log.stack(e,"SMTP");
                    sendErrorMessage(user,
                        "regarding '" + subject + "' because:\n\n" +
                        e.getMessage() +
                        "\n\nPlease fix the problem and try again.\n\n");
                }
            }

            if(recipHash.size() == 0)
                return false;
            InternetAddress[] recipIA = new InternetAddress[recipHash.size()];
            Iterator it = recipHash.iterator();
            for(int i=0;it.hasNext();i++) {
                recipIA[i] = (InternetAddress)it.next();
//                System.out.println(recipIA[i]);
            }

            javax.mail.Message msg = new javax.mail.internet.MimeMessage(
                Session.getDefaultInstance(System.getProperties(),null));
            msg.setFrom(new InternetAddress(from));
            msg.setRecipients(javax.mail.Message.RecipientType.TO,recipIA);
            msg.setSubject(subject);
            msg.setSentDate(new Date());
            msg.setText(message);
            Transport.send(msg);
        } catch(Exception e) {
            Log.stack(e,"SMTP");
            sendErrorMessage(user,
                        "regarding '" + subject + "' because:\n\n" +
                        e.getMessage() +
                        "\n\nPlease fix the problem and try again.\n\n");
            return false;
        }
        return true;
    }

    /**
    * Apumetodi virheilmoitusviestien lhettmiseksi.
    *
    * @param user kyttjolio jolle viesti lhetetn
    * @param body osa viestin bodysta
    */
    public static void sendErrorMessage(converge.kernel.User user, String body) {
        converge.service.Message m = new converge.service.Message(user);
        m.setMessageAttribute("From","The Friendly Converge System Administrators",true);
        m.setMessageAttribute("To","You",true);
        m.setMessageAttribute("Subject","[CONVERGE] Error sending email",true);
        m.setMessageAttribute("Date",new Date(),true);
        m.setMessageAttribute("MESSAGE_BODY",
            "Hi. This is the Converge System at University of Helsinki, Department\n" +
            "of Computer Science. I'm afraid I wasn't able to send your email\n" +
            body +
            "-- \nThe Friendly Converge System Administrators",true);
        converge.kernel.ContextModelHandler.incomingMessage(m);
    }

//DEBUG****************************
    public static void main(String[] args) {
        sendMail("avijohan@cc.helsinki.fi","testi1@converge.miuku.net,testi2@converge.miuku.net, avijohan@cc.helsinki.fi, virheell/{[}{}][!!inenosoite.com","testi",
        "Testiviestin body.\nToinen rivi\n\n-- \nConvergence of Messaging v0.000000000001",null);
    }
//DEBUG****************************
}

