import java.io.*;
/**
 * Program to count Lines Of Code(LOC) from the given source code file
 *
 * @author Mikko Hiipakka
 */

// VERSION CONTROL
//
// LocCounter.java
//
// Created On : 2002-13-10 13.30
// Created By : Mikko Hiipakka
//
// Last Modified On : 2002-13-10 17.00
// Last Modified By : Mikko Hiipakka
//



public class LocCounter {

  /** Logical Lines Count */
  private int loc = 0;
  /** Empty Lines Count */
  private int empty = 0;
  /** Comment Lines Count */
  private int comment = 0;



  /** Constructor */
  public LocCounter( String inputFile ){
  	try{
		process( inputFile );
	}catch(Exception e){
		e.printStackTrace();
		System.exit(0);
	}
  }


  /**  Accessory for instance's count of LOC */
  public int getLoc () {
    return this.loc;
  }

  /**  Accessory for instance's count of Empty lines */
  public int getEmptyLines () {
    return this.empty;
  }

  /**  Accessory for instance's count of Comment lines */
  public int getCommentLines () {
    return this.comment;
  }




  /** Private counter to process the given source code */
  private void process(String inputFile) throws Exception {
    String line = null;
    BufferedReader input = null;
    try{
      input = new BufferedReader( 
                 new InputStreamReader( 
                    new FileInputStream( inputFile ) ) );
    }catch( IOException e ){
      System.out.println("Target file not found");
      System.exit(0);
    }
    /*
     * Input file is read one line at a time and 
     * then the type of the line is determined and
     * corresponding counter is updated
     */
    while ( (line = input.readLine()) != null){

      line = line.trim();
      // Is the line empty 
      if ( line.length() == 0 ){
        this.empty++;
        continue;
      }
	  
      // Single line comment
      else if ( line.startsWith("//") || ( line.startsWith("/*") && line.endsWith("*/") ) ){
        this.comment++;
        continue;
	  }
	  // Multi line comment
	  else if ( line.startsWith("/*") ){
	  	do{
		  this.comment++;
		}while(!(line = input.readLine() ).endsWith("*/"));
		this.comment++;
		continue;
	  // Line is logical code line
      }else
        ++ this.loc;
      }
  }

  // main method. Program usage : java LocCounter "Target File Path"
  public static void main ( String [] args ){
    if ( args == null || args.length <= 0 ){
      System.out.println("Syntax error");
      System.out.println("Start by : java LocCounter \"Target File Path\"");
      return;
    }
    try{
      LocCounter counter = new LocCounter( args[0] );
      System.out.println("Target File Line Counts : ");
      System.out.println("Logical Lines : " + counter.getLoc());
      System.out.println("Empty Lines   : " + counter.getEmptyLines());
      System.out.println("Comment Lines : " + counter.getCommentLines());
    }catch( Exception e ){
       e.printStackTrace();
    }
  }
}
