/*
Tiedosto: Attachment.java
Luokat:   Attachment

Tekij:   Anssi Johansson
Luonti:   13.12.2002

Muutokset:
  13.12.2002  Luokan luonti

*/

package webcore;
import webcore.webserver.*;
import converge.kernel.*;
import converge.clientmanager.*;
import converge.service.*;
import java.io.*;
import java.util.*;
import org.w3c.dom.*;

/**
* Attachment huolehtii liitetiedostojen nyttmisest asiakkaalle
*/

public class Attachment implements WebRequestable {

    static ClientManager cm = ClientManager.getInstance();

    /**
    * Ksittelee sivupyynnn.
    * @param httpRequest pyynt
    */
    public void processRequest(HTTPRequest httpRequest) {
        System.out.flush();
        User user;
        long receivedSessionId=0;
        boolean hasCookie = false;
        boolean emergencyExit = false;


        String tmpCookie = httpRequest.getCookie("session-id");

        if(tmpCookie != null) {
            hasCookie = true;
            receivedSessionId = Long.parseLong(tmpCookie);
        }
        user = cm.getUserBySessionId(receivedSessionId);

        // jos ei ole validia user-oliota,
        // loggaudutaan ulos virheilmoituksen kera
        if (user == null)
            emergencyExit = true;

        try {

            if(emergencyExit || httpRequest.getFormVar("message") == null || httpRequest.getFormVar("id") == null) {

                HtmlUtils.printEmergencyExitPage(httpRequest.getPrintWriter());
            }
            else {

                String msgId = httpRequest.getFormVar("message");
                int attId = Integer.parseInt(httpRequest.getFormVar("id"));
                converge.service.Message msg = new converge.service.Message(user);
                Document dom = ViewMonitor.getMessage(user, msgId);

                if (dom != null) {
                    msg.loadXML(dom);
                    converge.service.Attachment[] atts = msg.getAttachmentList();
                    if(attId >= 0 && attId < atts.length) {
                        converge.service.Attachment a = atts[attId];
                        converge.service.AttachmentAttributes aa = a.getAttachmentAttributes();
                        PipeMessage pm = a.getAttachment(user);

                        if(pm.getReply() != null && !(pm.getReply().equals(""))) {
                            errorMessage(httpRequest.getPrintWriter(),pm.getReply(),user);
                            return;
                        }
                        final int bufsize = 32*1024;
                        int readbytes = 0;
                        char[] buffer = new char[bufsize];

                        OutputStream os = httpRequest.getOutputStream();
                        OutputStreamWriter osw = new OutputStreamWriter(os);
                        BufferedWriter bufw = new BufferedWriter(osw,bufsize);

                        String header = "";
                        if(aa.getSize()>0)
                            header = HtmlUtils.printHeader(
                                aa.getMIMEType(),aa.getSize(),aa.getFilename());
                        else
                            header = HtmlUtils.printHeader(
                                aa.getMIMEType(),aa.getFilename());
                        bufw.write(header,0,header.length());

                        InputStream is = pm.getPipe();
                        InputStreamReader isr = new InputStreamReader(is);
                        BufferedReader bufr = new BufferedReader(isr,bufsize);

                        while((readbytes = bufr.read(buffer,0,bufsize))>0)
                            bufw.write(buffer,0,readbytes);

                        bufr.close();
                        isr.close();
                        is.close();

                        bufw.flush();
                        osw.flush();
                        os.flush();
                        bufw.close();
                        osw.close();
                        os.close();
                    } else {
                        try {
                            errorMessage(httpRequest.getPrintWriter(),
                                        "Invalid attachment id",user);
                        } catch(Exception e) {}
                    }
                }
            }
        }
        catch (Exception e) {
            try {
                e.printStackTrace(httpRequest.getPrintWriter());
            } catch(Exception ee) {}
        }
        finally {
            try {
                httpRequest.close();
            } catch (java.io.IOException ee) {
                    try {
                        ee.printStackTrace(httpRequest.getPrintWriter());
                    } catch(Exception eee) {}
            }
        }
    }

    public void errorMessage(PrintWriter pw, String error, User user) {
          pw.println(htmlHeader);
          pw.print(HtmlUtils.startPage("Error"));
          pw.println("<BODY>");

          // 'navigaatiopalkki'
          HtmlUtils.printNavigationBar(pw);
          pw.print("<p>Kyttjtunnus: <b>"+user.getName()+"</b></p>\n");
          pw.print("<br><br><div align='center'><b>Error:</b>"
                    +error+"</div></body></html>");
    }

}
