/*
Tiedosto: ContextModel.java
Luokat:   ContextModel

Tekij:   Joni Karppinen
Luonti:   4.12.2002


Muutokset:
  4.12.2002 Joni
              Luokan luonti
  11.12.2002
            Olli
              Kontekstimallien luonti ja poisto
              Arvojen asettaminen malleihin
*/

package webcore;
import converge.kernel.*;
import converge.clientmanager.*;
import converge.service.*;
import java.io.*;
import java.util.*;
import webcore.webserver.*;
import converge.attribute.*;

/**
Kontekstimallien luominen, muuttaminen ja poistaminen.
 
Tm luokka toteuttaa WWW-kyttliittymn sen osan, jossa kyttj voi ksitell kontekstimallejaan.
 
<br><br>


*/
public class ContextModel implements WebRequestable{
    static ClientManager cm = ClientManager.getInstance();
    User user;
    long receivedSessionId;
    boolean hasCookie;
    boolean emergencyExit;

    /**
    Ksitelln HTTP-pyynt.
  
      @param httpRequest ksiteltv HTTP-pyynt
      */    
    public void processRequest(HTTPRequest httpRequest){
        PrintWriter pw = null;
        String tmpCookie = httpRequest.getCookie("session-id");
        if (tmpCookie != null){
            hasCookie = true;
            receivedSessionId = Long.parseLong(tmpCookie);
        }
        user = cm.getUserBySessionId(receivedSessionId);
        
        // jos ei ole validia user-oliota,
        // logataan ulos ja annetaan virheilmoitus
        if (user == null)            
            emergencyExit = true;
            
        else if (httpRequest.getFormVar("add-contextmodel") != null) {
            String cmName = httpRequest.getFormVar("add-contextmodel");
            if (user.writeLock(this)) {
              cmName = HtmlUtils.normalize(cmName);
              user.addContextModel(new converge.kernel.ContextModel(cmName));
              user.releaseLock(this);
            }
        }
        else if (httpRequest.getFormVar("delete-contextmodel") != null) {
            String cmName = httpRequest.getFormVar("delete-contextmodel");
            if (user.writeLock(this)) {
                user.deleteContextModel(cmName);
                user.releaseLock(this);
            }
        } 
        else if (httpRequest.getFormVar("model") != null &&
            httpRequest.getFormVar("attribute") != null  &&
            httpRequest.getFormVar("expression") != null &&
            httpRequest.getFormVar("attribute-value") != null &&
            httpRequest.getFormVar("weight") != null) {
                String modelName = httpRequest.getFormVar("model");
                if (user.writeLock(this)) {
                    converge.kernel.ContextModel model = user.getContextModel(modelName);
                    int weight = 0;
                    try {
                        weight= Integer.parseInt(httpRequest.getFormVar("weight"));
                    } catch(Exception e) {
                        model = null;
                    }
                    if (model != null) {
                        boolean forced = false;
                        if (httpRequest.getFormVar("forced") != null)
                            forced = true;
                        model.addAttribute(HtmlUtils.normalize(httpRequest.getFormVar("attribute")), 
                           HtmlUtils.normalize(httpRequest.getFormVar("expression")),
                           HtmlUtils.normalize(httpRequest.getFormVar("attribute-value")),
                           forced, weight);
            
                    }
                    user.releaseLock(this);
                }
        }
        
        
        try {
          
          pw = httpRequest.getPrintWriter();
          if (emergencyExit)
             HtmlUtils.printEmergencyExitPage(pw);
        
          else{
              pw.println(htmlHeader);
              pw.print(HtmlUtils.startPage("ContextModels"));
              pw.println("<BODY>");
          
          // 'navigaatiopalkki'
          HtmlUtils.printNavigationBar(pw);
          
          // pw.println("<h4><b>" + user.getName() + "&apos;s</b> contextmodels</h4>\n");
          //pw.print("<p>Logged in as: <b>"+user.getName()+"</b></p>\n");
          //pw.print("<h3>Context models</h3>\n");
          HtmlUtils.printPageInfo(pw, user.getName(), "Context models");
      
      
          if (httpRequest.getFormVar("model") != null && user.readLock(this)) {
            String modelName = httpRequest.getFormVar("model");
            converge.kernel.ContextModel model = user.getContextModel(modelName);
            if (model != null) {
              ContextModelAttributeInterface[] cmas = model.getAllAttributes();
        pw.println("<h4>" + model.getModelName() + "</h4>");
              pw.println("<table>");
              pw.println("<tr><td><b>Name</b></td><td>&nbsp;</td>" + 
                         "<td><b>Value</b></td><td><b>weight</b></td>" + 
                         "<td><b>forced</b></td><td></td></tr>");
              for (int i = 0; i < cmas.length; ++i) {
                pw.println("<tr><td>");
                pw.println(cmas[i].getName());
                pw.println("</td><td>");
                pw.println(cmas[i].getExpressionType());
                pw.println("</td><td>");
                pw.println((String)cmas[i].getValue());
                pw.println("</td><td>" + model.getAttributeWeight(cmas[i]) +"</td><td>" + 
                                model.isForcedAttribute(cmas[i]) + "</td><td></td></tr>");
              }
              pw.println("<form method='post' action='webcore.ContextModel'>");
              pw.println("<input type='hidden' name='model' value='" + modelName + "'>");
              pw.println("<tr><td><select name='attribute'>");
              String[] known = AttributeLoader.getKnownContextModelAttributes();
              for (int i = 0; i < known.length; ++i)
                pw.println("<option value='" + known[i] + "'>" + known[i]);
              pw.println("</select></td>");
              pw.println("<td><select name='expression'>" +
                       "<option value='in'>in" + 
                       "<option value='!in'>not in" + 
                       "<option value='='>=" + 
                       "<option value='!='>!=" + 
                       "</select></td>");
              pw.println("<td><input type=text name='attribute-value'></td>");
              pw.println("<td><input type=text name='weight' size='3'></td>");
              pw.println("<td><input type='checkbox' name='forced'></td>");
              pw.println("<td><input type=submit name='add' value='add'></td></tr>");
              pw.println("</form>");
              pw.println("</table>");
            }
            user.releaseLock(this);
          }
          else {
              if (user.readLock(this)) {
            converge.kernel.ContextModel[] cms = user.getContextModels();
            for (int i = 0; i < cms.length; ++i) {
              pw.println("<form method='post' action='webcore.ContextModel'>");
              pw.println("<a href='webcore.ContextModel?model=" 
                                            +cms[i].getModelName() 
                                            + "'>" + cms[i].getModelName() + "</a>");
              pw.println("<input type='hidden' name='delete-contextmodel' value='" 
                                            + cms[i].getModelName() + "'>");
              pw.println("<input type='submit' name='delete' value='delete'>");
              pw.println("</form>");
              pw.println("<br>");
            }
            user.releaseLock(this);
          }
          pw.println("<form method='post' action='webcore.ContextModel'>");
          pw.println("<input type=text name='add-contextmodel'>");
          pw.println("<input type=submit name='Add ContextModel'" + 
                                            " value='Add Context model'><br>");
          pw.println("<p>(Work, Freetime, ...)</p><br>");
          pw.println("</form>");
          }
        
            //testausta varten:
            HtmlUtils.printTestingStuff(pw, httpRequest);
            
            // sivun loppu:
            HtmlUtils.printEndOfPage(pw);
      }

        }
        catch (Exception e) {
            e.printStackTrace(pw);
        }
        finally {
            try{httpRequest.close();}
                catch (java.io.IOException e2) {e2.printStackTrace(pw);}
        }
        
    if (user != null)
        user.releaseLock(this);
    }

}
