/*
Tiedosto: Login.java
Luokat:   Login

Tekij:   Joni Karppinen
Luonti:   3.12.2002


Muutokset:

6.12.2002    Joni     
            Sisnkirjautumislomake alustavasti tehty, ei viel yhdistetty ytimeen 
9.12.2002    Joni    
            Lis testailua + toiminnallisuutta
14.12.2002   Olli
            Uuden kyttjn luonti, ryhmsisnkirjautuminen            
17.12.2002   Joni
            Javadocit, muut kommentit, siistimist
*/

package webcore;
import converge.kernel.*;
import converge.clientmanager.*;
//import converge.service.*;
import webcore.webserver.*;
import java.io.PrintWriter;


/**
Etusivu, sisnkirjautumisen ksittely, uusien kyttjien luonti.
<br><br>

    
*/
public class Login implements WebRequestable{

    static ClientManager cm = ClientManager.getInstance();

    User user;    // tss voidaan sivua vsttess silytt viitett siihen User-olioon,
                // jolle sivua tehdn

    boolean hasCookie = false;        // oliko http-pyynnn mukana session-id -keksi
    long receivedSessionId;            // keksist saatu session-id
    boolean createNewUser = false;    // ollaanko luomassa uutta (saadaan suoraan http-parametrista)
    boolean triedToLogin = false;
    boolean group = false;
    String username;
    String password;
    
    boolean successfulLogin = false;    // onnistuiko sisnkirjautuminen (keksi tai user/passwd)
    boolean setCookie = false;            // pitk asettaa keksi
    boolean loginForm = true;            // tulostetaanko login-form
    long newSessionId;
    boolean sessionExpired;
    boolean logout = false;
    
    /**
    Ksitelln HTTP-pyynt.
    
    @param httpRequest ksiteltv HTTP-pyynt
    */ 

    public void processRequest(HTTPRequest httpRequest){    

        PrintWriter pw = null;
        try {
            
            
            // Parametrien/keksien selvitys, asettaminen muuttujiin
            // ----------------------------------------------------
            
            // onko http-pyynnss mukana cookie?
            // (session-id == 0 tai session-id == "deleted"
            //  ksitetn samaksi tapaukseksi kuin se, ett ei saatu keksi)
            
            String tmpCookie = httpRequest.getCookie("session-id");
              
            if (tmpCookie != null && (!tmpCookie.equals("deleted")) 
                                  && (Long.parseLong(tmpCookie) > 0)){
                hasCookie = true;
                receivedSessionId = Long.parseLong(tmpCookie);
            }
            
            username = HtmlUtils.normalize(httpRequest.getFormVar("user"));
            password = httpRequest.getFormVar("passwd");
            if (httpRequest.getFormVar("logintype") != null &&
                httpRequest.getFormVar("logintype").equals("create"))
                createNewUser = true;
            if (httpRequest.getFormVar("logintype") != null &&
                httpRequest.getFormVar("logintype").equals("group"))
                group = true;
            if (httpRequest.getFormVar("logout") != null)
                logout = true;            
            if (httpRequest.getFormVar("login") != null)
                triedToLogin = true;    
            
            
            // User-olion haku, muu logiikka
            // -----------------------------

            if (hasCookie){
                user = cm.getUserBySessionId(receivedSessionId);
                if (user != null) {
                    successfulLogin = true;
                    loginForm = false;
                    setCookie = false;
                }
            }            
            if(username != null && password != null && !successfulLogin){
                newSessionId = cm.login(username, password, createNewUser, group);
                if (newSessionId > 0) {
                    user = cm.getUserBySessionId(newSessionId);
                    if (user != null){
                        successfulLogin = true;
                        loginForm = false;
                        setCookie = true;
                    }                    
                }
                else{
                    successfulLogin = false;
                }
            }
            
            if(hasCookie && !successfulLogin && !logout){    // sessio vanhentunut
                sessionExpired = true;
            }
            
            if (logout && hasCookie){
                cm.logout(receivedSessionId);
                loginForm = true;
            }
            
                        
            // HTML-sivun tulostus
            // -------------------
            
            pw = httpRequest.getPrintWriter();            
            
            // 1) HTTP-headerit            
            
            if (setCookie)
                pw.print(HtmlUtils.cookieHeader(newSessionId));
                                
            else if ((logout && hasCookie) || sessionExpired)
                pw.print(HtmlUtils.removeCookieHeader());
            
            else    
                pw.print(htmlHeader);
            
            // 2) sivun alku
            
            if(successfulLogin && !logout){
                printStartOfPage(pw, "Main");
                pw.print("<h4>Login in progress...</h4>\n");
                pw.print("<p>...redirecting to main page</p>");
                pw.print("<p>Username: <b>"+user.getName()+"</b></p>\n");
            }
            else        
                printStartOfPage(pw, null);
                                        
            
            // 3) mahdolliset (virhe)ilmoitukset & sisnkirjautumislomake
            
            if(hasCookie && logout)
                pw.print("<p>Logged out successfully.</p>\n");
            
            
            if (!successfulLogin && triedToLogin){
                // eli: yritettiin logata sisn, mutta ei onnistunut                
                pw.print("<p class='error'>Login failed</p>\n\n");
                        
            }
            if (sessionExpired)
                    pw.print("<br>Session has expired.\n");                
            
            if (loginForm)            
                printLoginForm(pw);
            
            // 4) testausjuttuja ja sivun loppu

            HtmlUtils.printTestingStuff(pw, httpRequest);            
            HtmlUtils.printEndOfPage(pw);
            
        } catch (Exception e) {
        
            //e.printStackTrace(pw);
            //DEBUG
            //System.out.println("webcore.Login: " + e);
            e.printStackTrace();
            //DEBUG
        }
        finally {
        
            try{httpRequest.close();}
                catch (java.io.IOException e2) {e2.printStackTrace(pw);}
                
       }

    }
              
       /**
       Pieni apumetodi, joka tulostaa sisnkirjautumislomakkeen tuottavan HTML:n annettuun PrintWriteriin.        
       */
       private static void printLoginForm(PrintWriter pw){
           pw.print("\n\n<fieldset style='margin-top: 1em; width: 350px;'>\n"
                +"<legend>Login</legend>\n\n"
                +"<form method='POST' action='webcore.Login'>\n"
                +"<table> <tr><td>Name:</td><td> <input type='text' name='user'></td></tr>\n"
                +"<tr><td>Password:</td><td><input type='password' name='passwd'></td></tr>\n"
                +"<tr><td><input type='hidden' value='yes' name='login'>\n"
            
                +"<tr><td colspan=2 align='center'>"
                +"<input type='radio' name='logintype' value='login' checked='checked'>User &nbsp; &nbsp; \n"
                +"<input type='radio' name='logintype' value='group'>Group &nbsp; &nbsp;\n"
                +"<input type='radio' name='logintype' value='create'>New User\n"
                +"</td></tr>\n<tr><td></td><td align='right'>"
                
                  +"<input type='submit' value='Sign in'> <input type='reset'></td></tr>\n"
                +"</table></form></fieldset>\n\n");
       }
       
       
       /** 
       Apumetodi, joka tulostaa sivun alkuosan. 
       @param pw Kytettv PrintWriter
       @param redirect Sen luokan (eli sivun) nimi jolle uudelleenohjataan
       */
         private static void printStartOfPage(PrintWriter pw, String redirect){

                if (redirect == null)
                pw.print(HtmlUtils.startPage("Login"));  
            else     
                pw.print(HtmlUtils.startPage("Login", redirect));              
                
            pw.print("<body>\n");
              pw.print("<H1>Converge Login</H1>\n");
        }
            

}
