/*
Tiedosto: Profile.java
Luokat:   Profile

Tekij:   Joni Karppinen
Luonti:   3.12.2002


Muutokset:
  3.12.  Joni
        Luokan luonti
  4.12.  Joni
        Hahmoteltu kommentteihin luokan toiminnallisuutta
  11.12. Olli
        Profiilien lisys ja poisto, sntjen lisys ja poisto                      
*/

package webcore;
import converge.kernel.*;
import converge.clientmanager.*;
import converge.service.*;
import java.io.*;
import java.util.*;
import webcore.webserver.*;

/**
Profiilien lisys, poisto ja muokkaus.

Toiminnallisuutta:<br>
 -profiilien luonti<br>
 -profiilien poistaminen<br>
 -profiilien muuttaminen (= profiilin sisltmien sntjen muuttaminen)<br>
<br>
+ nihin liittyvien HTML-sivujen generointi<br>

Esimerkiksi profiilin luonti:<br>

 -luodaan uusi Profile -olio<br>
 - listn siihen sntj.<br>
 - asetetaan aktiiviseksi <br>   

*/
public class Profile implements WebRequestable{
  static ClientManager cm = ClientManager.getInstance();
  User user;
  long receivedSessionId;
  boolean hasCookie;
  boolean emergencyExit;
  
  
  /**
  *Ksitelln http-pyynt.
  *
  */
    public void processRequest(HTTPRequest httpRequest){
      PrintWriter pw = null;
    
    // aluksi:  haetaan cookie, sen perusteella user
    
    String tmpCookie = httpRequest.getCookie("session-id");
    if (tmpCookie != null){
      hasCookie = true;
      receivedSessionId = Long.parseLong(tmpCookie);
    }
    user = cm.getUserBySessionId(receivedSessionId);
    
    if (user == null)
      emergencyExit = true;
    else if (httpRequest.getFormVar("add-profile") != null)
      this.addProfile(user, httpRequest);
    else if(httpRequest.getFormVar("delete-profile") != null)
      this.deleteProfile(user, httpRequest);
    else if (httpRequest.getFormVar("add-condition") != null && 
             httpRequest.getFormVar("rule-name") != null)
      this.addConditionToRule(user, httpRequest);
    else if (httpRequest.getFormVar("add-consequence") != null &&
             httpRequest.getFormVar("rule-name") != null)
      this.addConsequenceToRule(user, httpRequest);
    else if (httpRequest.getFormVar("delete-rule") != null)
      this.deleteRule(user, httpRequest);
    else if (httpRequest.getFormVar("add-contextmodel") != null && 
             httpRequest.getFormVar("limitvalue") != null)
      this.addContextModelValue(user, httpRequest);
    else if (httpRequest.getFormVar("new-scheduler") != null)
      this.addScheduler(user, httpRequest);
    else if (httpRequest.getFormVar("delete-scheduler") != null)
      this.deleteScheduler(user, httpRequest);
    else if (httpRequest.getFormVar("delete-scheduledrule") != null)
      this.deleteScheduledRule(user, httpRequest);
    else if (httpRequest.getFormVar("add-condition") != null && 
             httpRequest.getFormVar("scheduledrule-name") != null)
      this.addScheduledCondition(user, httpRequest);
    else if (httpRequest.getFormVar("add-consequence") != null && 
             httpRequest.getFormVar("scheduledrule-name") != null)
      this.addScheduledConsequence(user, httpRequest);
    else if (httpRequest.getFormVar("activate-profile") != null)
      this.activateProfile(user, httpRequest.getFormVar("activate-profile"), true);
    else if (httpRequest.getFormVar("deactivate-profile") != null)
      this.activateProfile(user, httpRequest.getFormVar("deactivate-profile"), false);
    
      
    
    
    try {
      pw = httpRequest.getPrintWriter();
      if (emergencyExit)
        HtmlUtils.printEmergencyExitPage(pw);
      else {
        pw.println(htmlHeader);
        pw.print(HtmlUtils.startPage("Profiles"));
        pw.println("<BODY>");

        // 'navigaatiopalkki'
        HtmlUtils.printNavigationBar(pw);
    
        // Sivun otsikko ja kyttjnimi jolla on kirjauduttu jrjestelmn        
        //pw.print("<p>Logged in as: <b>"+user.getName()+"</b></p>\n");
        //pw.print("<h3>Profiles</h3>\n");            
        HtmlUtils.printPageInfo(pw, user.getName(), "Profiles");
      
        
        if (httpRequest.getFormVar("scheduledrule-name") != null) {
          this.printScheduledRule(user, httpRequest, pw);
        
        }
        else if (httpRequest.getFormVar("profile") != null &&
            httpRequest.getFormVar("scheduler") != null) {
          this.printScheduledRules(user, httpRequest, pw);
        } 
        else if (httpRequest.getFormVar("profile") != null && 
            httpRequest.getFormVar("rule-name") != null)
          this.printRuleEdit(user, httpRequest, pw);
        else if (httpRequest.getFormVar("profile") != null && 
           httpRequest.getFormVar("rule-name") == null)
           this.printProfile(user, httpRequest, pw);
        else {          
          if (user.readLock(this)) {
            pw.println("<table>");
            converge.kernel.Profile[] ps = user.getProfiles();
            for (int i = 0; i < ps.length; ++i) {
              pw.println("<form method='post' action='webcore.Profile'>");
              pw.println("<tr><td><input type='hidden' name='delete-profile' value='" + 
              ps[i].getName() + "'>");
              pw.println("<a href='webcore.Profile?profile=" + ps[i].getName() + "'>" 
                                                    + ps[i].getName() + "</a>"); 
              pw.println("</td><td>");
              if (!ps[i].isDefaultProfile())
                pw.println("<input type='submit' name='delete' value='delete'>");
              else
                pw.println("&nbsp;");
              pw.println("</form></td><td>");
              pw.println("<form method='post' action='webcore.Profile'>");
              if (!(ps[i].isActive())) {
                pw.println("<input type='hidden' name='activate-profile' value='" 
                                                    + ps[i].getName() + "'>");
                pw.println("<input type='submit' name='activate' value='activate'");
                if ( ps[i].getContextModels().length == 0)
                  pw.println("disabled");
                pw.println(">");
              }
              else if (!ps[i].isDefaultProfile()){
                pw.println("<input type='hidden' name='deactivate-profile' value='" 
                                                    + ps[i].getName() + "'>");
                pw.println("<input type='submit' name='deactivate' value='deactivate'>");
              }
              if (!ps[i].isDefaultProfile() && ps[i].getContextModels().length == 0) {
                pw.println("<span style='color:red;'>No contextmodel references!</span>");
              }
              pw.println("</td></tr>");
              pw.println("</form>");
            }
          pw.println("</table>");
          user.releaseLock(this);
        }
        pw.println("<form method='post' action='webcore.Profile'>");
        pw.println("<input type='text' name='add-profile'>" + 
           "<input type='submit' name='Add Profile' value='Add Profile'><br>");
        pw.println("<p>(Work, Freetime...)</p><br>");
        pw.println("</form>");
      }
    }
    
    //testausta varten:
    HtmlUtils.printTestingStuff(pw, httpRequest);
    
    // sivun loppu:
    HtmlUtils.printEndOfPage(pw);
    
    } 
    catch (Exception e) {
      Log.stack(e, "webcore");
    }
    finally {
      try {
        httpRequest.close();
      }catch (Exception e) {
      Log.stack(e, "webcore");
      }
    
    }
    //Varmuudeksi vapautetaan mahdolliset lukot
    if (user != null)
        user.releaseLock(this);
    }
  
  /**
  *Poistetaam snt ajastimesta.
  *
  */
  private void deleteScheduledRule(User user, HTTPRequest httpRequest) {
    if (user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        Scheduler sche = profile.getScheduledRule(httpRequest.getFormVar("scheduler"));
        if (sche != null) {
          sche.deleteRule(httpRequest.getFormVar("delete-scheduledrule"));
        }
      }
      user.releaseLock(this);
    }
  }
  
  
  /**
  *Poistetaan ajastin.
  *
  */
  private void deleteScheduler(User user, HTTPRequest httpRequest) {
    if (user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null)
        profile.deleteScheduledRule(httpRequest.getFormVar("delete-scheduler"));
    user.releaseLock(this);
    }
  }
  
  /**
  *Listn ajastin profiiliin.
  *
  */
  private void addScheduler(User user, HTTPRequest httpRequest) {
    if (user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        String schedName = HtmlUtils.normalize(httpRequest.getFormVar("new-scheduler"));
        //Toistuva ajastus
        if (httpRequest.getFormVar("multiply") != null) {
          try {
            int multiply = Integer.parseInt(httpRequest.getFormVar("multiply"));
            String time = httpRequest.getFormVar("time");
            Scheduler sch = null;
            if (time.equals("minute"))
              sch = new Scheduler(schedName, Scheduler.MINUTE, multiply);
            else if(time.equals("hour"))
              sch = new Scheduler(schedName, Scheduler.HOUR, multiply);
            else if(time.equals("day"))
              sch = new Scheduler(schedName, Scheduler.DAY, multiply);
            else if(time.equals("week"))
              sch = new Scheduler(schedName, Scheduler.WEEK, multiply);
            else if(time.equals("month"))
              sch = new Scheduler(schedName, Scheduler.MINUTE, multiply);
            if (sch != null)
              profile.addScheduledRule(sch);
          }catch (Exception e) {
            Log.stack(e);
          }
        }
        //Kerran tapahtuva ajastus
       /* else if (httpRequest.getFormVar("year") != null){
          try {
            int year = Integer.parseInt(httpRequest.getFormVar("year"));
            int month = Integer.parseInt(httpRequest.getFormVar("month"));
            int day = Integer.parseInt(httpRequest.getFormVar("day"));
            ...
          }catch (Exception e) {
            Log.stack(e);
          }
        }*/
      }
      user.releaseLock(this);
    }
  }
  
  
  /**
  * Tulostetaan profiilin ajastimet.
  *
  */
  private void printSchedulers(User user, converge.kernel.Profile profile, PrintWriter pw) {
    pw.println("<hr>");
    pw.println("<b>Schedulers:</b><br>");
    Scheduler[] srules = profile.getScheduledRules();
    for (int i = 0; i < srules.length; ++i)
      if (!srules[i].exact()) {
        pw.println("<form method='post' action='webcore.Profile'>");
        pw.println("<input type='hidden' name='profile' value='" + profile.getName() +"'>");
        pw.println("<input type='hidden' name='delete-scheduler' value='" + srules[i].getName() +"'>");
        pw.println("<a href='webcore.Profile?profile="+ profile.getName() +"&scheduler=" + srules[i].getName() + "'>");
        switch (srules[i].getType()) {
          case Scheduler.MINUTE:
            pw.println(srules[i].getName() + ", every " + srules[i].getMultiply() + ". minutes");
            break;
          case Scheduler.HOUR:
            pw.println(srules[i].getName() + ", every " + srules[i].getMultiply() + ". hours");
            break;
          case Scheduler.DAY:
            pw.println(srules[i].getName() + ", every " + srules[i].getMultiply() + ". days");
            break;
          case Scheduler.WEEK:
            pw.println(srules[i].getName() + ", every " + srules[i].getMultiply() + ". weeks");
            break;
          case Scheduler.MONTH:
            pw.println(srules[i].getName() + ", every " + srules[i].getMultiply() + ". months");
            break;
          default:
            continue;
        }
        pw.println("</a>");
        pw.println("<input type=submit name='delete' value='delete'></form><br>"); 
      }
      
      
      pw.println("<form method='post' action='webcore.Profile'>");
      pw.println("<input type='hidden' name='profile' value='" + profile.getName() +"'>");
      pw.println("Scheduler name: <input type='text' name='new-scheduler'>");
      pw.println(" check every ");
      pw.println("<select name='multiply'>" + 
                   "<option value='1'>1" + 
                   "<option value='3'>3" + 
                   "<option value='5'>5" + 
                   "<option value='10'>10" + 
                   "<option value='15'>15" + 
                   "<option value='30'>30" + 
                   "<option value='45'>45" + 
                   "</select>");
      pw.println("<select name='time'>" + 
                   "<option value='minute'>minutes" + 
                   "<option value='hour'>hours" + 
                   "<option value='day'>days" + 
                   "<option value='week'>weeks" + 
                   "<option value='month'>months" + 
                   "</select>");
      pw.println("<input type=submit name='add' value='add'><br>");    
      pw.println("</form>");
      
      //Ei viel toteutettu, vaikkakin Scheduler ymmrt mys pivmri
      /*
      pw.println("<form method='post' action='webcore.Profile'>");
      pw.println("<input type='hidden' name='profile' value='" + profile.getName() +"'>");
      pw.println("Scheduler name: <input type='text' name='new-scheduler'>");
      pw.println(" check ");
      pw.println("<select name='year'>" + 
                   "<option value='2002'>2002" + 
                   "<option value='2003'>2003" + 
                   "</select>");
      pw.println("<select name='month'>" + 
                   "<option value='0'>Jan" + 
                   "<option value='1'>Feb" + 
                   "<option value='2'>Mar" + 
                   "<option value='3'>Apr" +
                   "<option value='4'>May" +
                   "<option value='5'>Jun" +
                   "<option value='6'>Jul" +
                   "<option value='7'>Aug" +
                   "<option value='8'>Sep" +
                   "<option value='9'>Oct" +
                   "<option value='10'>Nov" +
                   "<option value='11'>Dec" +
                   "</select>");
      pw.println("<select name='day'>");
      for (int i = 1 ; i < 32 ; ++i)
        pw.println("<option value='" + i +"'>" + i + "");
      pw.println("</select>");
      pw.println("<select name='hour'>");
      for (int i = 0 ; i < 24 ; ++i)
        pw.println("<option value='" + i +"'>" + i + "");
      pw.println("</select>");
      pw.println("<select name='minute'>");
      for (int i = 0 ; i < 60 ; ++i)
        pw.println("<option value='" + i +"'>" + i + "");
      pw.println("</select>");
      pw.println("<input type=submit name='add' value='add'><br>");    
      pw.println("</form>");
      */
  }

  
  
  /**
  *Tulostetaan ajastimen sntjen nimet.
  *
  */
  private void printScheduledRules(User user, HTTPRequest httpRequest, PrintWriter pw) {
    if (user.readLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        Scheduler sche = profile.getScheduledRule(httpRequest.getFormVar("scheduler"));
        if (sche != null) {
            pw.println("<h4>Scheduler: " + sche.getName() +"</h4>");
            pw.println("Rules:<br>");
            pw.println("<table>");
            String[] rules = sche.getRuleNames();
            for (int i = 0; i < rules.length; ++i)
              pw.println("<tr><td><a href='webcore.Profile?profile=" + 
                                      profile.getName() + "&scheduledrule-name=" + 
                                      rules[i] + "&scheduler=" + sche.getName() + "'>" + 
                                      rules[i] + "</a></td><td>" + 
                                      "<form method='post' action='webcore.Profile'>" +
                                      "<input type='hidden' name='profile' value='" + 
                                      httpRequest.getFormVar("profile") +"'>"+
                                      "<input type=hidden name='delete-scheduledrule' value='" + 
                                      rules[i] + "'>" +
                                      "<input type='hidden' name='scheduler' value='" + 
                                      sche.getName() +"'>"+
                                      "<input type=submit name='delete' value='delete'>"
                                      +"</form></td></tr>");
            pw.println("</table><br>");
            pw.println("<form method='post' action='webcore.Profile'>");
            pw.println("<input type=hidden name='profile' value='" + profile.getName() + "'>");
            pw.println("<input type=hidden name='scheduler' value='" + sche.getName() + "'>");
            pw.println("<input type=text name='scheduledrule-name'>");
            pw.println("<input type=submit name='Add Rule' value='Add Rule'><br>");
            pw.println("</form>");
        
        
        }
      }
      user.releaseLock(this);
    }
  }
  
  
  /**
  *Tulostetaan ajastimen snt.
  *
  *
  */
  private void printScheduledRule(User user, HTTPRequest httpRequest, PrintWriter pw) {
    if(user.readLock(this)) {
      boolean lastWasOperator = true;
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      Scheduler sche = null;
      if (profile != null)
        sche = profile.getScheduledRule(httpRequest.getFormVar("scheduler"));
      if (sche != null) {
        ScheduledRule rule = sche.getRule(httpRequest.getFormVar("scheduledrule-name"));
        //Rule on jo kannassa
        if (rule != null) {
          pw.println("<h4>" + rule.getName() +"</h4>");
          List conditions = rule.getConditions();
          Iterator it = conditions.iterator();
          pw.println("Conditions: <br>");
          while (it.hasNext()) {
            Object cond = it.next();
            if (cond instanceof Triple) {
              Triple t = (Triple)cond;
              pw.println(t.getName() + " " + t.getExpression() + " " + t.getValue() + "<br>");
              lastWasOperator = false;
            }
            else if (cond instanceof ConditionalOperator) {
              pw.println(((ConditionalOperator)cond).getType());
              lastWasOperator = true;
            }
          }
         
          List consequences = rule.getConsequences();
          it = consequences.iterator();
          pw.println("Consequences:<br>");
          while (it.hasNext()) {
            Consequence c = (Consequence) it.next();
            if (c.getType() == Rule.EVENT_ADD) {
              pw.println("ADD: " + c.getName() + " : " + c.getValue());
            }
            else if (c.getType() == Rule.EVENT_DELETE) {
              if (c.getValue() == null)
                pw.println("DELETE: " + c.getName());
             else
               pw.println("DELETE: " + c.getName() + " : " + c.getValue());
             }
             pw.println("<br>");
           }
         }
         pw.println("<hr>");
         //Snnn jatkaminen
         pw.println("Add condition:<br>");
         pw.println("<form method='post' action='webcore.Profile'>");
         pw.println("<input type='hidden' name='add-condition' value='true'>");
         pw.println("<input type='hidden' name='profile' value='" + 
                                    httpRequest.getFormVar("profile") +"'>");
         pw.println("<input type='hidden' name='scheduler' value='" + 
                                    sche.getName() +"'>");
         pw.println("<input type='hidden' name='scheduledrule-name' value='" + 
                                    httpRequest.getFormVar("scheduledrule-name") +"'>");
         if (!lastWasOperator)
           pw.println("<select name='operator'>" + 
                     "<option value='AND'>AND" + 
                    "<option value='OR'>OR" + 
                    "</select>");
           
           
         pw.println("<select name='attribute'>");
         String[] known = converge.attribute.AttributeLoader.getKnownUserProperties();
         for (int i = 0; i < known.length; ++i) {
           pw.println("<option value='" + known[i] + "'>" + known[i]);
         }
         pw.println("</select>");
         pw.println("<select name='expression'>" +
                 "<option value='='>=" + 
                 "<option value='!='>!=" + 
                 "<option value='&lt;'>&lt;" + 
                 "<option value='&gt;'>&gt;" + 
                 "</select>");
         pw.println("<input type='text' name='value'>");
         pw.println("<input type=submit name'add' value='add'>");
         pw.println("</form>");
          
          
         pw.println("Add consequence:<br>");
         pw.println("<form method='post' action='webcore.Profile'>");
         pw.println("<input type='hidden' name='add-consequence' value='true'>");
         pw.println("<input type='hidden' name='profile' value='" + 
                                      httpRequest.getFormVar("profile") +"'>");
         pw.println("<input type='hidden' name='scheduler' value='" + 
                                      sche.getName() +"'>");
         pw.println("<input type='hidden' name='scheduledrule-name' value='" + 
                          httpRequest.getFormVar("scheduledrule-name") +"'>");
         pw.println("<select name='type'>" + 
               "<option value='ADD'>ADD" + 
               "<option value='DELETE'>DELETE" + 
               "</select>");
         pw.println("<select name='action'>");
         known = ActionLoader.getKnownActions();
         for (int i = 0; i < known.length; ++i) {
           ActionInterface ai = ActionLoader.newActionInstance(known[i]);
           if (ai != null && ai.supportsScheduledRule(known[i]) == true)
             pw.println("<option value='" + known[i] + "'>" + known[i]);
         }
         pw.println("</select>");
         pw.println("<input type='text' name='value'>");
         pw.println("<input type=submit name='add' value='add'>");
         pw.println("</form>");
           
       }
     user.releaseLock(this);
    }
  }
  
  
  
  /**
  *Listn ajastimen sntn ehto.
  *
  */
  private void addScheduledCondition(User user, HTTPRequest httpRequest) {
    if (user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      Scheduler sche = null;
      if (profile != null)
        sche = profile.getScheduledRule(httpRequest.getFormVar("scheduler"));
      if (sche != null) {
        ScheduledRule rule = sche.getRule(httpRequest.getFormVar("scheduledrule-name"));
      if (rule == null)
        rule = new ScheduledRule(HtmlUtils.normalize(httpRequest.getFormVar("scheduledrule-name")));
      if (httpRequest.getFormVar("operator") != null)
        rule.nextCondition(httpRequest.getFormVar("operator"));
      String name = httpRequest.getFormVar("attribute");
      String expression = httpRequest.getFormVar("expression");
      String value = HtmlUtils.normalize(httpRequest.getFormVar("value"));
      if (value != null && name != null && expression != null) {
        rule.addCondition(name, value, expression);
      }
      if (name != null)
        sche.setRule(rule);
      }
      user.releaseLock(this);
    }
  }
  
  
  /**
  *Listn ajastimen sntn toiminto.
  *
  */
  private void addScheduledConsequence(User user, HTTPRequest httpRequest) {
    if(user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      Scheduler sche = null;
      if (profile != null)
        sche = profile.getScheduledRule(httpRequest.getFormVar("scheduler"));
      if (sche != null) {
        ScheduledRule rule = sche.getRule(httpRequest.getFormVar("scheduledrule-name"));
      if (rule == null)
        rule = new ScheduledRule(HtmlUtils.normalize(httpRequest.getFormVar("scheduledrule-name")));
      int type = Rule.EVENT_ADD;
      if (httpRequest.getFormVar("type").equals("DELETE"))
        type = Rule.EVENT_DELETE;
      String action = httpRequest.getFormVar("action");
      String value = HtmlUtils.normalize(httpRequest.getFormVar("value"));
      if (!(value == null && type == Rule.EVENT_ADD)) {
        rule.consequence(type, action, value);
        sche.setRule(rule);
      }
      }
    user.releaseLock(this);
    }
  }
  
  
  /**
  *Aktivoidaan profiili.
  *
  *
  */
  private void activateProfile(User user, String name, boolean activate) {
    converge.kernel.Profile profile = user.getProfile(name);
    if (profile != null && !(profile.isDefaultProfile()))
      profile.setActive(activate);
    
  }
  
  
  /**
  *Listn profiiliin kontekstimallin kynnysarvo.
  *
  */
  private void addContextModelValue(User user, HTTPRequest httpRequest) {
     if (user.writeLock(this)) {
      int limit = 0;
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      try {
         limit = Integer.parseInt(httpRequest.getFormVar("limitvalue"));
      }
       catch (Exception e) {
        Log.stack(e);
       }
      if (limit < 0)
        limit = 0;
      if (limit > 100 && profile != null)
        profile.removeContextModelValue(httpRequest.getFormVar("add-contextmodel"));
      else if (profile != null) {
        profile.setContextModelValue(httpRequest.getFormVar("add-contextmodel"), limit);
      }
      user.releaseLock(this);
    }
  }
  
  
  /**
  *Poistetaan snt profiilista.
  *
  *
  */
  private void deleteRule(User user, HTTPRequest httpRequest) {
    if (user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        profile.deleteRule(httpRequest.getFormVar("delete-rule"));
      }
      user.releaseLock(this);
    } 
  }
  
  
  /**
  *Listn sntn toiminto.
  *
  *
  */
  private void addConsequenceToRule(User user, HTTPRequest httpRequest) {
    if(user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
          Rule rule = profile.getRule(httpRequest.getFormVar("rule-name"));
        if (rule == null)
          rule = new Rule(HtmlUtils.normalize(httpRequest.getFormVar("rule-name")));
        int type = Rule.EVENT_ADD;
        if (httpRequest.getFormVar("type").equals("DELETE"))
          type = Rule.EVENT_DELETE;
        String action = httpRequest.getFormVar("action");
        String value = HtmlUtils.normalize(httpRequest.getFormVar("value"));
        if (!(value == null && type == Rule.EVENT_ADD)) {
          rule.consequence(type, action, value);
          profile.setRule(rule);
        }
      }
      user.releaseLock(this);
    }
  }
  
  
  
  /**
  *Listn sntn ehto.
  *
  */
  private void addConditionToRule(User user, HTTPRequest httpRequest) {
    if(user.writeLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        Rule rule = profile.getRule(httpRequest.getFormVar("rule-name"));
        if (rule == null)
        rule = new Rule(HtmlUtils.normalize(httpRequest.getFormVar("rule-name")));
        if (httpRequest.getFormVar("operator") != null)
          rule.nextCondition(httpRequest.getFormVar("operator"));
        String name = httpRequest.getFormVar("attribute");
        String expression = httpRequest.getFormVar("expression");
        String value = HtmlUtils.normalize(httpRequest.getFormVar("value"));
        if (value != null && name != null && expression != null) {
          rule.addCondition(name, value, expression);
        }
        profile.setRule(rule);
      }
      user.releaseLock(this);
    } 
  }
  
  
  /**
  *Poistetaan profiili.
  *
  */
  private void deleteProfile(User user, HTTPRequest httpRequest)  {
    String profName = httpRequest.getFormVar("delete-profile");
    //Default-tarkistuksen voisi hoitaa paremmin kysymll 
    //profiililta isDefaultProfile
    if (!profName.equals(converge.kernel.Profile.DEFAULT) && user.writeLock(this)) {
      user.deleteProfile(user.getProfile(profName));
      user.releaseLock(this);
    }
  }
  
  
  /**
  *Listn uusi profiili tai korvataan vanha uudella.
  */
  private void addProfile(User user, HTTPRequest httpRequest) {
    String profName = HtmlUtils.normalize(httpRequest.getFormVar("add-profile"));
    if (!profName.equals(converge.kernel.Profile.DEFAULT) && user.writeLock(this)) {
      user.addProfile(new converge.kernel.Profile(user, profName));
      user.releaseLock(this);
    }
  }
  
  
  /**
  * Tulostetaan snnnmuokkauslomake.
  */
  private void printRuleEdit(User user, HTTPRequest httpRequest, PrintWriter pw) {
    if(user.readLock(this)) {
      boolean lastWasOperator = true;
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
         Rule rule = profile.getRule(HtmlUtils.normalize(httpRequest.getFormVar("rule-name")));
         //Rule on jo kannassa
         if (rule != null) {
           pw.println("<h4>" + rule.getName() +"</h4>");
           
           
           List conditions = rule.getConditions();
           Iterator it = conditions.iterator();
           pw.println("Conditions: <br>");
           while (it.hasNext()) {
             Object cond = it.next();
             if (cond instanceof Triple) {
               Triple t = (Triple)cond;
               pw.println(t.getName() + " " + t.getExpression() + " " + 
                                                        t.getValue() + "<br>");
               lastWasOperator = false;
             }
             else if (cond instanceof ConditionalOperator) {
               pw.println(((ConditionalOperator)cond).getType());
               lastWasOperator = true;
             }
           }
           
           
           List consequences = rule.getConsequences();
           it = consequences.iterator();
             pw.println("Consequences:<br>");
             while (it.hasNext()) {
               Consequence c = (Consequence) it.next();
               if (c.getType() == Rule.EVENT_ADD) {
                 pw.println("ADD: " + c.getName() + " : " + c.getValue());
               }
               else if (c.getType() == Rule.EVENT_DELETE) {
                 if (c.getValue() == null)
                   pw.println("DELETE: " + c.getName());
                 else
                   pw.println("DELETE: " + c.getName() + " : " + c.getValue());
               }
               pw.println("<br>");
             }
         }
         
         pw.println("<hr>");
         //Snnn jatkaminen
         pw.println("Add condition:<br>");
         pw.println("<form method='post' action='webcore.Profile'>");
         pw.println("<input type='hidden' name='add-condition' value='true'>");
         pw.println("<input type='hidden' name='profile' value='" + 
              HtmlUtils.normalize(httpRequest.getFormVar("profile")) +"'>");
         pw.println("<input type='hidden' name='rule-name' value='" + 
              HtmlUtils.normalize(httpRequest.getFormVar("rule-name")) +"'>");
         if (!lastWasOperator)
           pw.println("<select name='operator'>" + 
                   "<option value='AND'>AND" + 
                   "<option value='OR'>OR" + 
                   "</select>");
            
         pw.println("<select name='attribute'>");
         String[] known = converge.attribute.AttributeLoader.getKnownMessageAttributes();
         for (int i = 0; i < known.length; ++i) {
           pw.println("<option value='" + known[i] + "'>" + known[i]);
          }
          pw.println("</select>");
          pw.println("<select name='expression'>" +
                   "<option value='in'>in" + 
                   "<option value='!in'>not in" + 
                   "<option value='='>=" + 
                   "<option value='!='>!=" + 
                   "</select>");
          pw.println("<input type='text' name='value'>");
          pw.println("<input type=submit name='add' value='add'>");
          pw.println("</form>");
            
            
          pw.println("Add consequence:<br>");
          pw.println("<form method='post' action='webcore.Profile'>");
          pw.println("<input type='hidden' name='add-consequence' value='true'>");
          pw.println("<input type='hidden' name='profile' value='" + 
                                    httpRequest.getFormVar("profile") +"'>");
          pw.println("<input type='hidden' name='rule-name' value='" + 
                                    httpRequest.getFormVar("rule-name") +"'>");
          pw.println("<select name='type'>" + 
                   "<option value='ADD'>ADD" + 
                   "<option value='DELETE'>DELETE" + 
                   "</select>");
          pw.println("<select name='action'>");
          known = ActionLoader.getKnownActions();
          for (int i = 0; i < known.length; ++i) {
            ActionInterface ai = ActionLoader.newActionInstance(known[i]);
            if (ai != null && ai.supportsRule(known[i]) == true)
              pw.println("<option value='" + known[i] + "'>" + known[i]);
          }
          pw.println("</select>");
          pw.println("<input type='text' name='value'>");
          pw.println("<input type=submit name='add' value='add'>");
          pw.println("</form>");
            
        }
        user.releaseLock(this);
     }
  }
  
  
  /**
  *Tulostetaan profiili, sen sntjen ja ajastimien nimet ja kontekstimallien
  *kynnysarvot.
  *
  */
  private void printProfile(User user, HTTPRequest httpRequest, PrintWriter pw) {
    if(user.readLock(this)) {
      converge.kernel.Profile profile = user.getProfile(httpRequest.getFormVar("profile"));
      if (profile != null) {
        pw.println("<h4>" + profile.getName() +"</h4>");
        pw.println("Rules:<br>");
        pw.println("<table>");
        String[] rules = profile.getRuleNames();
        for (int i = 0; i < rules.length; ++i)
          pw.println("<tr><td><a href='webcore.Profile?profile=" + 
                          profile.getName() + "&rule-name=" + 
                          rules[i] + "'>" + rules[i] + "</a></td><td>" + 
                          "<form method='post' action='webcore.Profile'>" +
                          "<input type='hidden' name='profile' value='" + 
                          httpRequest.getFormVar("profile") +"'>"+
                          "<input type=hidden name='delete-rule' value='" + 
                          rules[i] + "'>" +
                          "<input type=submit name='delete' value='delete'>"
                         +"</form></td></tr>");
          pw.println("</table><br>");
          pw.println("<form method='post' action='webcore.Profile'>");
          pw.println("<input type=hidden name='profile' value='" + profile.getName() + "'>");
          pw.println("<input type=text name='rule-name'>");
          pw.println("<input type=submit name='Add Rule' value='Add Rule'><br>");
          pw.println("</form>");
            
          if (!profile.isDefaultProfile()){
            pw.println("<hr>");
            pw.println("<table><tr><td>ContextModel</td><td>Limit value</td></tr>");
            String[] cmvs = profile.getContextModels();
            for (int i = 0; i < cmvs.length; ++i) {
              pw.println("<tr><td>" + cmvs[i] + "</td><td>" + profile.getContextModelValue(cmvs[i]) + "%</td></tr>");
            }
            pw.println("</table>");
            converge.kernel.ContextModel[] models = user.getContextModels();
            if (models.length > 0) {
              pw.println("<form method='post' action='webcore.Profile'>");
              pw.println("<input type=hidden name='profile' value='" + profile.getName() + "'>");
              pw.println("<select name='add-contextmodel'>");
              for (int i = 0; i < models.length; ++i)
                pw.println("<option value='" +models[i].getModelName() + "'>" + models[i].getModelName()); 
              pw.println("</select>");
              pw.println("<input type=text name='limitvalue' size='3'>");
              pw.println("<input type='submit' name='Add ContextModel Value' value='Add ContextModel Value'><br>");
              pw.println("</form>");
            }
          }
        this.printSchedulers(user, profile, pw);
      }
      user.releaseLock(this);
    }
  }
}
