/*
Tiedosto: Settings.java
Luokat:   

Tekij:   Joni Karppinen
Luonti:   10.12.2002


Muutokset:
10.12.2002  Joni  Luokan luonti + alustava toteutus
16.12.2002    Joni  Olemassaolevien datasource-tietojen tarkempi listaus ym.
*/

package webcore;
import converge.kernel.*;
import converge.clientmanager.*;
import converge.service.*;
import java.io.*;
import java.util.*;
import webcore.webserver.*;


/**
Asetuksien muuttaminen. Tss toteutuksessa vain koskee vain IMAP-palvelimien osoitteita, tunnuksia ja salasanoja.
<br><br>
[http-parametreja ainakin:<br>

imap-new
imap-server
imap-username
imap-passwd]

*/
public class Settings implements WebRequestable {

    static ClientManager cm = ClientManager.getInstance();
    User user;
    long receivedSessionId;
    boolean hasCookie;
    boolean emergencyExit;
  
    String[] dataSourceNames;
    String imapServer;
    String imapUsername;
    String imapPassword;
    String dataSourceName;
    String deleteName;
    
    /**
    Ksitelln HTTP-pyynt.
    
    @param httpRequest ksiteltv HTTP-pyynt
    */

    public void processRequest(HTTPRequest httpRequest) {
        PrintWriter pw = null;
    
        // aluksi:  haetaan cookie, sen perusteella user
        
        String tmpCookie = httpRequest.getCookie("session-id");
        if (tmpCookie != null){
            hasCookie = true;
            receivedSessionId = Long.parseLong(tmpCookie);
        }
        user = cm.getUserBySessionId(receivedSessionId);
        
        // jos ei ole validia user-oliota,
        // loggaudutaan ulos virheilmoituksen kera
        if (user == null)            
            emergencyExit = true;
    
    
    
    
        // uuden IMAP-datasourcen lisys:        
    
        if (httpRequest.getFormVar("imap-new") != null){
          imapServer = httpRequest.getFormVar("imap-server");
          imapUsername = httpRequest.getFormVar("imap-username");
          imapPassword = httpRequest.getFormVar("imap-passwd");
          dataSourceName = HtmlUtils.normalize(httpRequest.getFormVar("datasource-name"));
        }
        
        if (imapServer != null && imapUsername != null && imapPassword != null 
                               && dataSourceName != null && user.writeLock(this)) {
          //String[] dss = user.getDataSourceNames();
          //int i = dss.length + 1;
          System.err.println("Setting IMAP");
          IMAP newIMAP = new IMAP(dataSourceName, imapServer, imapUsername, imapPassword);
          user.addDataSource(newIMAP);      
          
          // + ajastettu snt joka hakee viestit tuolta
          // (vliaikainen testiviritys)
          /*int multiply = Integer.parseInt(httpRequest.getFormVar("multiply"));
          System.out.println("MULTIPLY" + multiply);
          String time = httpRequest.getFormVar("multiply");
          Scheduler sched;
          sched = new Scheduler(dataSourceName, Scheduler.MINUTE, multiply);
          if (time.equals("hour"))
            sched = new Scheduler(dataSourceName, Scheduler.HOUR, multiply);
          else if (time.equals("day"))
            sched = new Scheduler(dataSourceName, Scheduler.DAY, multiply);
          else if (time.equals("month"))
            sched = new Scheduler(dataSourceName, Scheduler.MONTH, multiply);
          
          ScheduledRule rule = new ScheduledRule("Hae viestit");
          rule.consequence(Rule.EVENT_ADD, "Get-Message", dataSourceName);
          sched.setRule(rule);
          converge.kernel.Profile prof = user.getDefaultProfile();
          prof.addScheduledRule(sched);*/
          user.releaseLock(this);
        }
        
        if (httpRequest.getFormVar("delete-datasource") != null  && 
                                              user.writeLock(this)) {
            deleteName = httpRequest.getFormVar("delete-datasource");
            user.deleteDataSource(deleteName);
            user.getDefaultProfile().deleteScheduledRule(deleteName);
            user.releaseLock(this);
        }
    
        try {
      
          pw = httpRequest.getPrintWriter();
    
          if (emergencyExit)
             HtmlUtils.printEmergencyExitPage(pw);
            
          else{
            pw.println(htmlHeader);
            pw.print(HtmlUtils.startPage("DataSources"));
            pw.println("<BODY>");
          
              // 'navigaatiopalkki'
            HtmlUtils.printNavigationBar(pw);
          
            // pw.print("<h4><b>" + user.getName() + "&apos;s</b> settings</h4><br>\n");
            //pw.print("<p>Logged in as: <b>"+user.getName()+"</b></p>\n");                
            //pw.print("<h3>Datasources</h3>\n");                
            
            // Sivun otsikko ja kyttjnimi jolla on kirjauduttu jrjestelmn
            HtmlUtils.printPageInfo(pw, user.getName(), "Datasources");
         
              //printataan olemassaolevat datasourcet:
          
              if (user.readLock(this)) {        
                dataSourceNames = user.getDataSourceNames();
                if (dataSourceNames.length > 0){
                    pw.println("<hr>");
                    pw.println("<h4>Existing datasources:</h4>\n");
                    pw.println("<table cellspacing='10'>");
                    pw.println("<tr><th>Name</th><th>Server</th><th>Username</th></tr>");
                }    
                for(int i=0; i<dataSourceNames.length; i++){
                    DataSource tmpDataSource = user.getDataSource(dataSourceNames[i]);
                    String tmpAddress = "", tmpLogin = "";    
                    if (tmpDataSource.getType().equals("IMAP")){
                        IMAP tmpIMAP = (IMAP) tmpDataSource;
                        tmpAddress = tmpIMAP.getServerAddress();
                        tmpLogin = tmpIMAP.getServerLogin();                
                    }
                    pw.println("<tr>");                    
                    pw.println("<form method='post' action='webcore.Settings'>");
                    pw.println("<input type='hidden' name='delete-datasource' value='" 
                        + dataSourceNames[i] + "'>");
                    pw.print("<td>" + dataSourceNames[i] + "</td>\n");
                    pw.print("<td>" + tmpAddress + "</td>\n");
                    pw.print("<td>" + tmpLogin + "</td>\n");
                    pw.print("<td>&nbsp;&nbsp;<input type='submit' name='delete' value='delete'></td>");
                    pw.println("</tr></form>");                  
                }
                if (dataSourceNames.length > 0){
                    pw.println("</table>");
                    pw.println("<hr>");            
                }
                 user.releaseLock(this);
            }
          
          /*if (user.readLock(this)) {
            pw.println("<hr>Oletusprofiilin ajastetut snnt: <br><br>");
            pw.println("<table>");
            Scheduler[] scs = user.getDefaultProfile().getScheduledRules();
            for(int i=0; i < scs.length; i++){
              pw.println("<tr><td>");
              pw.print(scs[i].getName()+
                  " </td></tr>");
                }
             pw.println("</table>");
            user.releaseLock(this);
          }*/

          
          
              // lomake jolla voi list uuden IMAP-datasourcen    
            
            pw.println("<h4>Add a new IMAP-datasource</h4>\n");          
            pw.println("<form method='post' action='webcore.Settings'>");
            pw.println("<input type=text name='datasource-name' value='IMAP1'>Name<br>");
            pw.println("<input type=text name='imap-server' value='imap.converge.miuku.net'>Server (IMAP)<br>");
            pw.println("<input type=text name='imap-username' value='testi1'>Username<br>");
            pw.println("<input type=password name='imap-passwd' value='converge'>Password<br>");
            /*pw.println("Check for new messages every ");
            pw.println("<select name='multiply'>" + 
                       "<option value='1'>1" + 
                       "<option value='3'>3" + 
                       "<option value='5'>5" + 
                       "<option value='10'>10" + 
                       "</select>");
            pw.println("<select name='time'>" + 
                       "<option value='minute'>minutes" + 
                       "<option value='hour'>hours" + 
                       "<option value='day'>days" + 
                       "<option value='month'>months" + 
                       "</select><br>");*/
            pw.println("<br><input type=hidden name='imap-new' value='true'> "
                    +"<input type=submit  value='Add new server'> <input type=reset><br>");    
            pw.println("</form>");
          
    

          
    
          //testausta varten:
          HtmlUtils.printTestingStuff(pw, httpRequest);
          
          // sivun loppu:
          HtmlUtils.printEndOfPage(pw);
          }
            }
    
            catch (Exception e) {
                e.printStackTrace(pw);
            }
    
            finally {
                try{httpRequest.close();}
                    catch (java.io.IOException e2) {e2.printStackTrace(pw);}
            }
        if (user != null)
            user.releaseLock(this);
    }



}
