

package webcore.webserver;

import java.io.*;
import java.util.Hashtable;

/****
 * Class with static methods to get the mime type of a file.
 * The mime type is determined by file extension and the table in
 * read into the class in the load method.<p>
 * The mime types are stored in a serialized Hashtable object which
 * is generated by MimeTypeSetup.
 * @see
 * MimeTypeSetup
 */
public class MimeType
{

    /***
     * hash table for mime types
     */
    private static Hashtable mimeTable;

    /**
     * reads the mime types from a resource
     * with a java.io.ObjectInputStream
     */
    public static void load() throws IOException
    {
        try
        {
            ObjectInputStream in = new ObjectInputStream(
                MimeType.class.getResource("mimetypes.dat").openStream());
            mimeTable = (Hashtable)in.readObject();
            in.close();
        }
        catch (Exception e)
        {
            throw new IOException(
                "Unable to read mimetable.dat file in webserver package.");
        }
    }


    /**
     * returns the mime type based on a file name.  The mime type is determined
     * by the extension of the file using the mime table.
     */
    public static String getMimeType(String fileName)
    {
        String extension = extractExt(fileName);
        String result = (String)mimeTable.get(extension);
        if (result == null) result = new String("text/plain");
        return result;
    }

    /***
     * Internally used method to extract the extension from a filename.
     */
    private static String extractExt(String path)
    {
        String result = new String(path);
        int i;
        while (result.indexOf(".") >= 0)
        {
            i = result.indexOf(".") + 1;
            result = result.substring(i, result.length());
        }
        return result;
    }
}