package webcore.webserver;

/**
 * Interface for a factory class which creates web requestables.  Each Webserver
 * Instance uses a WebRequestableFactoryInterface to construct WebRequstables
 * to handle requests.<p>
 * The WebRequestableFactoryInterface examine the HTTPRequest passed to the
 * factory method and return a WebRequestable which will be used by the
 * WebServer to process the HTTPRequest.
 *
 */

public interface WebRequestableFactoryInterface
{
    /**
     * Must return a WebRequestable which is suitable to process this
     * HTTPRequest.  The Webserver will call this method from a Threads in the
     * Webserver's Thread pool.<p>
     * Implementers of this method must provide a WebRequestable even for static
     * files. HTTPFileRetriever is recommended for the default WebRequestable
     * unless static files are not being served.
     */
    WebRequestable createWebRequestable(HTTPRequest request);
}
